#
# AggregateBrainImages.R
#
# This creates the figures summarizing the results after fitting the BLF to 
# each of the remaining AD patient brain images
################################################################################

rm(list= ls())

require(tidyverse)
require(reshape2)
require(gridExtra)

# Results directory
res_loc <- "Results/All_AD_Brains/"
dir_0592 <- "Results/0592MCMC_results.Rdata"  # The "single brain analysis"
  # results are stored elsewhere
all3d_dir <- "ADNI_data_3D/all3D/"  # Directory for the target files
img_dir <- "Images/ADNI_Study/"  # Address for the images


# AD patient labels
ad.brains <- c("0784", "0754", "0592", "0221", "1263", "0088")


################################################################################
  

# Iterate through each to store the results into a csv file so that they
# can be summarized
for (index in 1:length(ad.brains)){
  
  brain <- ad.brains[index]
  
  if (brain == "0592"){
    
    load(dir_0592)
    
  } else {
    
    load(paste0(res_loc, brain, "_MCMC_Results.Rdata"))
    
  }
  
  
  ####################
  
  # Get the other segmentations
  
  # Majority Voting
  maj.vot <- apply(segmentations, c(1,2,3), sum)
  maj.vot <- (maj.vot > dim(segmentations)[4]/2)
  
  
  # Globally-weighted majority vote
  gw.maj.vot1 <- array(0, c(P1, P2, P3))
  gw.maj.vot0 <- array(0, c(P1, P2, P3))
  for (i in 1:dim(segmentations)[4]){
    
    gw.maj.vot1 <- gw.maj.vot1 + segmentations[, , , i]/mean(sims[, , , i]^2)
    gw.maj.vot0 <- gw.maj.vot0 + (1-segmentations[, , , i])/mean(sims[, , , i]^2)
    
  }
  gw.maj.vot <- array(0, c(P1, P2, P3))
  for (i in 1:P1){
    for (j in 1:P2){
      for(k in 1:P3){
        
        gw.maj.vot[i,j,k] <- ifelse(gw.maj.vot1[i,j,k] > gw.maj.vot0[i,j,k], 1, 0)  
        
      }
    }
  }
  gw.maj.vot[gw.maj.vot1 == gw.maj.vot0] <- rbinom(1, 1, 0.5)
  
  
  # Locally weighted majority vote
  lw.maj.vot1 <- apply(segmentations/(sims^2), c(1,2,3), sum)
  lw.maj.vot0 <- apply((1-segmentations)/(sims^2), c(1,2,3), sum)
  lw.maj.vot <- array(0, c(P1, P2, P3))
  for (i in 1:P1){
    for (j in 1:P2){
      for (k in 1:P3){
        
        lw.maj.vot[i,j,k] <- ifelse(lw.maj.vot1[i,j,k] > lw.maj.vot0[i,j,k], 1, 0)   
        
      }
    }
  }
  lw.maj.vot[lw.maj.vot1 == lw.maj.vot0] <- rbinom(1, 1, 0.5)
  
  
  # Get the joint label fusion (JLF) result
  jlf.seg <- readNIfTI(paste0(all3d_dir, brain, "/", brain, 
                              "_jlf_segmentation_cn_to_ad.nii.gz"))
  
  
  # Get the manual segmentation
  tru.seg <- readNIfTI(paste0(all3d_dir, brain, "/", brain,
                              "_true_segmentation.nii.gz"))
  
  
  
  ##########################
  
  # Compute the volume estimates and the volume similarities relative to the
  # manual segmentation
  
  # To get the Bayesian LF volume estimate, pull out appropriate part of the 
  # MCMC chain so that the Bayesian LF volume estimates can be computed
  keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])
  prb.mean <- apply(probs.MCMC[keep, ], 2, mean)
  vol.ests.rb <- apply(probs.MCMC[keep, ], 1, sum)
  
  blf.vol <- mean(vol.ests.rb)
  blf.vol.c99 <- quantile(vol.ests.rb, probs= c(0.005, 0.995))  # Quantiles
  blf.l99 <- as.numeric(blf.vol.c99[1])
  blf.u99 <- as.numeric(blf.vol.c99[2])
  
  # Thresholded posterior probabilities to produce a binary map
  blf.est <- (array(prb.mean, c(P1, P2, P3)) > 0.5)*1
  
  
  # Volume estimates from the other binary segmentations
  mv.vol <- sum(maj.vot)
  gwmv.vol <- sum(gw.maj.vot)
  lwmv.vol <- sum(lw.maj.vot)
  jlf.vol <- sum(jlf.seg)
  man.vol <- sum(tru.seg)  # Manual volume estimate
  
  
  #################
  
  
  # Volume similarities
  tab.blf <- table(c(tru.seg), blf.est)
  tab.mv <- table(c(tru.seg), maj.vot)
  tab.gwv <- table(c(tru.seg), gw.maj.vot)
  tab.lwv <- table(c(tru.seg), lw.maj.vot)
  tab.jlf <- table(c(tru.seg), jlf.seg)
  
  vs_blf <- 1 - abs(tab.blf[2,1] - tab.blf[1,2])/(2*tab.blf[2,2] + 
                                                    tab.blf[1,2] + tab.blf[2,1])
  
  vs_jlf <- 1 - abs(tab.jlf[2,1] - tab.jlf[1,2])/(2*tab.jlf[2,2] + 
                                                    tab.jlf[1,2] + tab.jlf[2,1])
  
  vs_mv <- 1 - abs(tab.mv[2,1] - tab.mv[1,2])/(2*tab.mv[2,2] + 
                                                 tab.mv[1,2] + tab.mv[2,1])
  
  vs_gwv <- 1 - abs(tab.gwv[2,1] - tab.gwv[1,2])/(2*tab.gwv[2,2] + 
                                                    tab.gwv[1,2] + tab.gwv[2,1])
  
  vs_lwv <- 1 - abs(tab.lwv[2,1] - tab.lwv[1,2])/(2*tab.lwv[2,2] + 
                                                    tab.lwv[1,2] + tab.lwv[2,1])
  
  
  ##############################################################################
  
  # Write to file
  res.summary <- data.frame(brain= brain, 
                            jlf_vol= jlf.vol, 
                            blf_vol= blf.vol, 
                            blf_l99= blf.l99, 
                            blf_u99= blf.u99, 
                            vs_jlf= vs_jlf, 
                            vs_blf= vs_blf,
                            vs_mv= vs_mv, 
                            vs_gwv= vs_gwv, 
                            vs_lwv= vs_lwv, 
                            man_vol= man.vol)
  
  if (index == 1){
    
    write.table(res.summary, file= paste0(res_loc, "Ctrl2Ad_GMI_Summary.csv"),
                append= FALSE, sep= ",", row.names= FALSE, col.names= TRUE)
    
  } else {
    
    write.table(res.summary, file= paste0(res_loc, "Ctrl2Ad_GMI_Summary.csv"),
                append= TRUE, sep= ",", row.names= FALSE, col.names= FALSE)
    
  }  # End if

  
}  # End loop over patient labels


################################################################################

# Load the summary file and create the figure
rm(list= ls()[!(ls() %in% c('res_loc', 'img_dir'))])
res <- read.csv(paste0(res_loc, "Ctrl2Ad_GMI_Summary.csv"), header= TRUE)

# Volume Similarities
vs <- data.frame(Segmentation= rep(c("LWMV", "BLF", "JLF", "MV", "GWMV"), 
                           each= dim(res)[1]), 
                 VS= c(res$vs_lwv,
                         res$vs_blf,
                         res$vs_jlf,
                         res$vs_mv,
                         res$vs_gwv))

tru.vols <- res$man_vol

#################

# The reproduces Figure 12 in the manuscript

## The commented out code will produce a black and white version of the figure.
## The uncommented code produces a color figure via ggplot

# pdf(paste0(img_dir, "AggregateSummary.pdf"), height= 7, width= 14)
# par(mfrow= c(1,2))
# 
# # Plot the Bayesian LF, JLF, and manual volume estimates (other are left out
# # for cleanliness)
# plot(x= tru.vols, y= 1:dim(res)[1], pch= 4, xlim= c(2000, 7500), 
#      yaxt= "n", 
#      xlab= bquote("Estimated Volumes"~"(mm"^3~")"), ylab= "Subjects", lwd= 2, 
#      main= "Hippocampal Volumes", cex.axis= 1.5, cex.lab= 1.5, cex.main= 1.5)
# points(x= res$blf_vol, y= 1:(dim(res)[1]), pch= 6, lwd= 2)
# points(x= res$jlf_vol, y= 1:(dim(res)[1]), pch= 1, lwd= 2)
# segments(res$blf_l99, 1:(dim(res)[1]), res$blf_u99, 1:(dim(res)[1]), lwd= 2)
# legend(x= 6000, y= 2, legend= c("BLF (99% Cred. Int.)", "JLF", "Manual"), 
#        pch= c(6,1,4), cex= 0.75, bty= "n")
# axis(side= 2, at= 1:(dim(res)[1]), labels= FALSE)
# 
# 
# boxplot(vs$diff ~ vs$type, xlab= "Method", ylab= "Volume Similarity",
#         main= "Volume Similarities",
#         ylim= c(0,1), cex.axis= 1.5, cex.lab= 1.5, cex.main= 1.5)
# 
# dev.off()



temp_df <- res[, c("jlf_vol", "blf_vol", "man_vol")]
names(temp_df) <- c("JLF", "BLF", "Manual")
vol_df <- melt(temp_df)
vol_df$y_loc= rep(1:6, times= 3)
names(vol_df) <- c("Segmentation", "Volume", "Subjects")

ci_df <- data.frame(x= res$blf_l99, y= 1:(dim(res)[1]), 
                    xend= res$blf_u99, yend= 1:(dim(res)[1]))


# Only include BLF, JLF, and manual in the volume plot
plot1 <- ggplot(vol_df, aes(Volume, Subjects)) +
          geom_segment(data= ci_df, aes(x, y, xend= xend, yend= yend), 
                       color= "chartreuse3", size= 1.25) +
          geom_point(aes(color= Segmentation, shape= Segmentation), size= 2) +
          coord_cartesian(xlim= c(2000, 7500)) +
          labs(title= "Hippocampal Volumes", 
               x = bquote("Estimated Volumes"~"(mm"^3~")"),
               y = "Subjects") +
          theme(axis.text.y = element_blank(),
                axis.ticks.y = element_blank(),
                plot.title= element_text(size= 21, hjust= 0.5),
                axis.title= element_text(size= 14))
  

plot2 <- ggplot(vs, aes(x= Segmentation, y= VS)) +
          geom_boxplot() +
          coord_cartesian(ylim= c(0, 1)) +
          labs(title = "Volume Similarities", x = "Segmentation", 
               y = "Volume Similarity") +
          theme(plot.title= element_text(size= 21, hjust= 0.5),
                axis.title= element_text(size= 14))


# This produces Figure 12 in the manuscript
pdf(paste0(img_dir, "AggregateSummary_Pretty.pdf"), height= 7, width= 14)
grid.arrange(plot1, plot2, nrow= 1)
dev.off()



################################################################################

# Re-load subject 1263 to show the trace plot of volume
rm(list= ls()[!(ls() %in% c('res_loc', 'img_dir'))])
brain <- "1263"
load(paste0(res_loc, brain, "_MCMC_Results.Rdata"))

# Look at last 2000 iterations
keep <- (length(lamj.SeR.MCMC[[1]])/3 + 1):length(lamj.SeR.MCMC[[1]])
vol.ests.rb <- apply(probs.MCMC[keep, ], 1, sum)

# This creates Supplementary Figure 7
pdf(paste0(img_dir, "1263VolTrace.pdf"))
plot(vol.ests.rb, type= "l", xlab= "Iteration", ylab= "Volume Estimate")
dev.off()