// [[Rcpp::depends(RcppArmadillo)]]
// [[Rcpp::depends(RcppTN)]]
// [[Rcpp::plugins(cpp11)]]
#define ARMA_64BIT_WORD 1
#include <RcppArmadillo.h>
#include <RcppTN.h>
using namespace arma;
using namespace Rcpp;

static double const log2pi = std::log(2.0 * M_PI);


// [[Rcpp::export]]
void set_seed(unsigned int seed) {
  Rcpp::Environment base_env("package:base");
  Rcpp::Function set_seed_r = base_env["set.seed"];
  set_seed_r(seed);  
}


// [[Rcpp::export]]
arma::mat mvrnormArma(int n, vec mu, mat sigma){
  // The is from gallery.rcpp.org/articles/simulate-multivariate-normal
  
  int ncols = sigma.n_cols;
  mat Y = arma::randn(n, ncols);
  
  return repmat(mu, 1, n).t() + Y * chol(sigma);
}  // End function



/* C++ version of the dtrmv BLAS function */
void inplace_tri_mat_mult(arma::rowvec &x, arma::mat const &trimat){
  arma::uword const n = trimat.n_cols;
  
  for(unsigned j = n; j-- > 0;){
    double tmp(0.);
    for(unsigned i = 0; i <= j; ++i)
      tmp += trimat.at(i, j) * x[i];
    x[j] = tmp;
  }
}


// [[Rcpp::export]]
arma::vec dmvnrm_arma_fast(arma::mat const &x,  
                           arma::rowvec const &mean,  
                           arma::mat const &sigma, 
                           bool const logd = false) { 
  using arma::uword;
  uword const n = x.n_rows, 
    xdim = x.n_cols;
  arma::vec out(n);
  arma::mat const rooti = arma::inv(trimatu(arma::chol(sigma)));
  double const rootisum = arma::sum(log(rooti.diag())), 
    constants = -(double)xdim/2.0 * log2pi, 
    other_terms = rootisum + constants;
  
  arma::rowvec z;
  for (uword i = 0; i < n; i++) {
    z = (x.row(i) - mean);
    inplace_tri_mat_mult(z, rooti);
    out(i) = other_terms - 0.5 * arma::dot(z, z);
  }  
  
  if (logd)
    return out;
  return exp(out);
}



// [[Rcpp::export]]
List sampleZj_C(List eta_SeR, List eta_SpR, vec Tees, mat Y_mat, int R, int V){
  // # Sampling latent Zj that arise from the Albert / Chib truncated normal
  // # sampling trick.
  // #
  // # eta.SeR = sensitivity spatial field (= phi_r in the paper)
  // # eta.SpR = specificity spatial field (= eta_r in the papar)
  // # Ts = current vector of true inclusion indicators
  // # Y = array of observed atlas segmentations
  // # R = number of atlas segmentations
  // # V = total number of voxels
  
  // Initialization
  List Zj_SeR(R);
  List Zj_SpR(R);
  unsigned int num_Tees = sum(Tees);
  unsigned int num_notTees = V - sum(Tees);
  arma::vec temp1(num_Tees), tempTrunc(num_Tees), tempTrunc2(num_notTees);
  arma::vec l_se;
  arma::vec u_se, tempY0, etaTemp; //(no_Tees, -99);
  arma::vec temp2(num_notTees), l_sp, u_sp, tempY1; //(no_notTees, -99);
  uvec pos, y_pos;
  //List out;
  temp1 = temp1 - 99;
  temp2 = temp2 - 99;
  

  for (int i = 0; i < R; i++){
    
    Zj_SeR[i] = temp1;
    Zj_SpR[i] = temp2;
    
  }  // End loop
  
  
  // Update step
  for (int r = 0; r < R; r++){
    
    l_se = temp1;
    u_se = temp1;
    l_sp = temp2;
    u_sp = temp2;
    
    
    // Update Ts == 1 (R code analogs are given for reference)
    // # Update latent sens. variables first.
    // # Truncation range for Y = 0 = (-infty, 0)
    //     l.se[Y[Ts==1,r] == 0] <- -Inf
    //       u.se[Y[Ts==1,r] == 0] <- 0
    //
    // # Truncation range for Y = 1 = (0, infty)
    //     l.se[Y[Ts==1,r] == 1] <- 0
    //     u.se[Y[Ts==1,r] == 1] <- Inf
    //
    // # Sensitivites <-> phi in the paper
    // #Zj.SeR[[r]]<-rtnorm(sum(Ts), eta.SeR[[r]][Ts==1], 1, lower=l.se, upper=u.se)
    //       Zj.SeR[[r]]<-rtruncnorm(n= sum(Ts), mean= eta.SeR[[r]][Ts==1],sd= 1,
    //                               a= l.se, b= u.se)
    pos = arma::find(Tees == 1);
    tempY1 = Y_mat.col(r);
    y_pos = arma::find(tempY1(pos) == 0);
    for(unsigned int i = 0; i < y_pos.size(); i++){
      
      l_se(y_pos(i)) = R_NegInf;
      u_se(y_pos(i)) = 0.0;
      
    } // End loop
    
    y_pos = arma::find(tempY1(pos) == 1);
    for(unsigned int i = 0; i < y_pos.size(); i++){
      
      l_se(y_pos(i)) = 0.0;
      u_se(y_pos(i)) = R_PosInf;
      
    }  // End loop
    
    etaTemp = as<vec>(eta_SeR[r])(pos);
    
    for (unsigned int i = 0; i < num_Tees; i++){
      
      tempTrunc(i) = RcppTN::rtn1(etaTemp(i), 1, l_se(i), u_se(i));
      
    }
    
    Zj_SeR[r] = tempTrunc;
    
    
    pos = arma::find(Tees == 0);
    tempY0 = Y_mat.col(r);
    y_pos = arma::find(tempY0(pos) == 0);
    for(unsigned int i = 0; i < y_pos.size(); i++){
      
      l_sp(y_pos(i)) = 0.0;
      u_sp(y_pos(i)) = R_PosInf;
      
    } // End loop
    
    y_pos = arma::find(tempY0(pos) == 1);
    for(unsigned int i = 0; i < y_pos.size(); i++){
      
      l_sp(y_pos(i)) = R_NegInf;
      u_sp(y_pos(i)) = 0.0;
      
    }  // End loop
    
    etaTemp = as<vec>(eta_SpR[r])(pos);
    
    for (unsigned int i = 0; i < num_notTees; i++){
      
      tempTrunc2(i) = RcppTN::rtn1(etaTemp(i), 1, l_sp(i), u_sp(i));
      
    }
    
    Zj_SpR[r] = tempTrunc2;
    
    
  }  // End R (rater) loop
  
  //return Zj_SeR;
  
  return List::create(_["Zj_SeR"] = Zj_SeR, _["Zj_SpR"] = Zj_SpR);
  
}  // End fun sampleZj_C





// [[Rcpp::export]]
List mt_ct_BCJ(arma::vec b, arma::vec y, arma::mat X, 
               arma::vec y_add, arma::vec n_add, 
               arma::mat X_add){
  // # Finds the Mean and Covariance of proposal density according to Gammerman 
  // # (1997)
  // #
  // # b= coefficient vector
  // # y= "observed" data
  // # X= "observed" design matrix
  // # Y.add= "additional" reponses
  // # n.add= weights (= no. trials) associated with Y.add
  // # X.add= design matrix associated with "additional" responses
  
  // int P = b.size();
  mat Xb = X*b;
  mat Xb_add = X_add*b;
  mat X_c;
  mat XWX;
  mat C;
  mat M;
  double L;
  
  vec p_vec;
  vec p_add_vec;
  vec w_vec;
  vec w_add_vec;
  vec yb_vec;
  vec yb_add_vec;
  
  
  p_vec = exp(Xb) / (1 + exp(Xb));
  w_vec = p_vec%(1 - p_vec);
  
  p_add_vec = exp(Xb_add) /  (1 + exp(Xb_add));
  w_add_vec = n_add%p_add_vec%(1 - p_add_vec);
  
  yb_vec = Xb%w_vec + (y - p_vec);
  yb_add_vec = Xb_add%w_add_vec + (y_add - n_add%p_add_vec);
  
  
  X_c = join_vert(X, X_add);
  w_vec = join_vert(w_vec, w_add_vec);
  yb_vec = join_vert(yb_vec, yb_add_vec);
  
  XWX = (diagmat(w_vec)*X_c).t()*X_c;
  C = inv(XWX);
  
  M = C*(X_c.t()*yb_vec);
  
  L = sum(y%Xb - log(1+exp(Xb))) + sum(y_add%(Xb_add) - n_add%log(1 + exp(Xb_add)));
  
  //return(L);
  return List::create( _["M"] = M, _["C"] = C, Named("L") = L);
  
}  // End function mt_ct_BCJ





// [[Rcpp::export]]
arma::vec MH_logistic_reg(vec b0, mat X, 
                          vec y, vec y_add, 
                          vec n_add, mat X_add){
  // # MH update for a logistic link following the Gamerman algorithm
  // # using the mean and cov created in mt_ct.u 
  // #
  // # b0 = current state of the coefficient vector
  // # X = observed design matrix
  // # y = observed (binary) responses
  // # Y.add = augmenting (binomial) responses
  // # n.add = weights (trials) associated with the augmenting responses
  // # X.add = design matrix associated with the augmenting responses
  
  List res;
  mat Mb0, Mb_star;
  mat Cb0, Cb_star;
  double Lb0, Lb_star, ratio_pi, test;
  vec b_star, b1;
  vec qb_star, qb0;
  
  
  
  res = mt_ct_BCJ(b0, y, X, y_add, n_add, X_add);
  Mb0 = Rcpp::as<mat>(res["M"]);
  Cb0 = Rcpp::as<mat>(res["C"]);
  Lb0 = res["L"];
  
  
  b_star = mvrnormArma(1, Mb0, Cb0).t();
  qb_star = dmvnrm_arma_fast(b_star.t(), Mb0.t(), Cb0, true);
  
  
  
  res = mt_ct_BCJ(b_star, y, X, y_add, n_add, X_add);
  Mb_star = Rcpp::as<mat>(res["M"]);
  Cb_star = Rcpp::as<mat>(res["C"]);
  Lb_star = res["L"];
  
  qb0 = dmvnrm_arma_fast(b0.t(), Mb_star.t(), Cb_star, true);
  
  
  //ratio.pi <- exp(Lb.star-Lb0+qb0-qb.star)
  ratio_pi = exp(Lb_star - Lb0 + qb0(0) - qb_star(0));  // Starst at 0!!!
  
  test = runif(1)(0);
  if(test <= ratio_pi){
    
    b1 = b_star;
    
  } else {
    
    b1 = b0;
    
  }  // End if
  
  
  return b1;
  
}  //End function MH_logistic_reg





// [[Rcpp::export]]
arma::vec FS_GMRF(arma::vec Z, double rho, arma::vec D, arma::mat Nbs,
                  arma::vec mu, arma::vec phi, double lam, arma::uvec id1,
                  arma::uvec id2, arma::uvec id3, arma::uvec id4, arma::uvec id5,
                  arma::uvec id6, arma::uvec id7, arma::uvec id8,
                  LogicalVector incls){
  
  // Note: This function is not used in our current implementation. But it is
  // included here for reference, and possible experimentation if a practioner
  // wants to try.
  
  arma::vec Yt = zeros(incls.size());
  arma::uvec q = as<arma::uvec>(incls);
  arma::uvec::iterator it;
  double mui, s2i, pm, ps, o1;
  
  
  
  Yt.elem(find(q)) = Z;  // Logical indexing in Yt
  
  //1
  for (it = id1.begin(); it != id1.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
 
  
  
  //2
  for (it = id2.begin(); it != id2.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  
  
  //3
  for (it = id3.begin(); it != id3.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  
  //4
  for (it = id4.begin(); it != id4.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  //5
  for (it = id5.begin(); it != id5.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  // 6
  for (it = id6.begin(); it != id6.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  //7
  for (it = id7.begin(); it != id7.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  //8
  for (it = id8.begin(); it != id8.end(); ++it){
    
    mui = 0.0;
    for (int j = 0; j < D[*it-1]; j++){
      
      mui = mui + rho*(phi[Nbs(*it-1, j) - 1] - mu[Nbs(*it-1,j) - 1]);
      
    }  // End inner loop
    mui = (D[*it-1]*mu[*it-1] + mui)/D[*it-1];
    s2i = 1.0/(lam*D[*it-1]);
    o1 = q[*it-1];
    pm = (s2i*o1*Yt[*it-1] + mui)/(s2i*o1 + 1);
    ps = sqrt(s2i/(1 + s2i*o1));
    
    phi[*it-1] = R::rnorm(pm, ps);
    
  }  // End loop over id's
  
  
  
  return phi;
  
}  // End function FS_GMRF





// [[Rcpp::export]]
arma::vec SingleSite(arma::vec Z, double rho, arma::vec D, arma::vec mu,
                     arma::mat Nbs, arma::vec phi, double lam, 
                     LogicalVector incls){
  // See the SingleSite cpp function from the TAS paper for this function's
  // inspiration.
  // Input:
  // Z - vector of latent auxiliary variables
  // rho - propriety parameter (double)
  // D - vector of CAR weights (Dw in the usual formulation). D[i] = # of 
  //    neighbors for voxel i
  // Nbs - Matrix of neighbors Nbs[i, j] = jth neighbor of voxel i
  // phi - current state of the random field (vector to be updated)
  // lam - CAR precision parameter (double)
  // incls - inclusion indicators (vector of Tv's)
  
  int i_size = incls.size();  // = V, total # of voxels
  arma::vec Yt = zeros(i_size);
  arma::uvec q = as<arma::uvec>(incls);
  double mu_i, s2i, p_m, p_s;
  
  Yt.elem(find(q)) = Z;  // Logical indexing in Yt
  
  // Iterate through the list and pull the neighbors for each voxel
  for (int i = 0; i < i_size; i++){
    
    mu_i = 0.0;
    
    for (int j = 0; j < D[i]; j++){
      
      mu_i = mu_i + rho*(phi[Nbs(i, j) - 1] - mu[Nbs(i,j) - 1]);
      
    }  // End inner loop
    mu_i = (D[i]*mu[i] + mu_i)/D[i];
    s2i = 1.0/(lam*D[i]);
    
    //pm <- as.vector((s2i*Yt[id1]*O1 + mui[id1])/(1 + s2i*O1))
    p_m = (s2i*incls[i]*Yt[i] + mu_i)/(s2i*incls[i] + 1);
    //ps <- as.vector(sqrt((s2i)/(1 + s2i*O1)))
    p_s = sqrt(s2i/(1 + s2i*incls[i]));
    
    phi[i] = R::rnorm(p_m, p_s);
    
  }  // End outer loop
  
  return phi;
  
}  // End fn SingleSite





// [[Rcpp::export]]
List Sample_Ts(arma::vec Xb, List eta_ser, List eta_spr, NumericMatrix Y, 
               int V, int R){
  // A basic Rcpp analog of the SampleTs fn from MCMCAuxEcon3D.R
  //
  // Input:
  // Xb - "X\beta" vector (mean function) from the MCMC
  // eta_ser - list of (untransformed) sensitivity fields
  // eta_spr - list of (untransformed) specificity fields
  // Y - V x R matrix of observed segmentations
  // R - integer # of raters
  
  arma::vec p_vec, q_vec(V), t_vec(V);
  double n = 1.0;
  double d = 1.0;
  NumericMatrix ser_p(V, R), spr_p(V, R), ser_p2(V, R), spr_p2(V, R);
  
  
  p_vec = exp(Xb) / (1 + exp(Xb));
  
  for (int j = 0; j < R; j++){
    
    ser_p(_, j) = pnorm(as<NumericVector>(eta_ser[j]));
    ser_p2(_, j) = 1.0 - ser_p(_, j);
    
    spr_p(_, j) = pnorm(as<NumericVector>(eta_spr[j]));
    spr_p2(_, j) = 1.0 - spr_p(_, j);
    
  }
  
  for (int i = 0; i < V; i++){
    
    for (int j = 0; j < R; j++){
      
      ser_p(i, j) = pow(ser_p(i,j), Y(i,j));
      ser_p2(i, j) = pow(ser_p2(i,j), 1.0 - Y(i, j));
      n = n*ser_p(i, j)*ser_p2(i, j);
      
      spr_p(i, j) = pow(spr_p(i,j), 1.0 - Y(i,j));
      spr_p2(i, j) = pow(spr_p2(i,j), Y(i,j));
      d = d*spr_p(i, j)*spr_p2(i, j);
      
    }  // End j loop
    
    q_vec[i] = p_vec[i]*n / (p_vec[i] * n + (1.0 - p_vec[i]) * d);
    t_vec[i] = R::rbinom(1, q_vec[i]);
    
    n = 1.0;
    d = 1.0;
    
  }  // End i loop
  
  //return List::create(ser_p, ser_p2, spr_p, spr_p2);
  return List::create( _["Ts"] = t_vec, _["prob"] = q_vec);
  
  
}  // End function Sample_Ts





// [[Rcpp::export]]
List Sample_Lamj(List alam_ser, List alam_spr, List blam_ser, List blam_spr,
                 List eta_ser, List eta_spr, List mu_ser, List mu_spr, 
                 arma::sp_mat DjWj, int R, int V){
  // ######################################################
  // # This function updates the precision parameters (lam_j's) for the spatial
  // # processes. It just follows from straightforward conditional conjugacty.
  // #
  // # alam.SeR = prior shape parameter for the sens. precisions (these can be
  // #   atlas-specific)
  // # alam.SpR = prior shape parameter for the spec. precisions
  // # blam.SeR = prior rate parameter for the sens. precisions
  // # blam.SpR = prior rate parameter for the spec. precisions
  // # eta.SeR = spatial fields associated with the sens. processes
  // # eta.SpR = spatial fields associated with the spec. processes
  // # mu.SeR = mean functions for the sens. spatial processes
  // # mu.SpR = mean functions of the specs. spatial processes
  // # DjWj = Dj-rho*Wj (= precision matrix for the CAR models)
  // # R = number of atlas segmentations
  // # V = number of voxels
  
  List alam_u_ser(R), alam_u_spr(R), blam_u_ser(R), blam_u_spr(R), lamj_ser(R),
  lamj_spr(R);
  arma::vec cntrd(V);
  arma::vec qf;
  
  for(int r = 0; r < R; r++){
    
    alam_u_ser[r] = as<double>(alam_ser[r]) + V/2.0;  // Can be created outside
    alam_u_spr[r] = as<double>(alam_spr[r]) + V/2.0;
    
    cntrd = as<arma::vec>(eta_ser[r]) - as<arma::vec>(mu_ser[r]); 
    
    qf = (cntrd.t() * (DjWj*cntrd));
    
    blam_u_ser[r] = as<double>(blam_ser[r]) + qf(0)/2.0;  // Can be recycled
    blam_u_spr[r] = as<double>(blam_spr[r]) + qf(0)/2.0;
    
    lamj_ser[r] = R::rgamma(alam_u_ser[r], 1.0/as<double>(blam_u_ser[r]));
    lamj_spr[r] = R::rgamma(alam_u_spr[r], 1.0/as<double>(blam_u_spr[r]));
    
  }  // End loop over raters
  
  return List::create(_["lamj_SeR"] = lamj_ser, _["lamj_SpR"] = lamj_spr);
  
  
}  // End function Sample_Lamj 
  
  