#
# BadSegSimulations.R
#
# This simulates the data and atlases for the 'bad but in agreement' simulation
# study in the manuscript.
################################################################################

rm(list= ls())

require(mvtnorm)
require(spatstat)
require(lattice)
require(gridExtra)

set.seed(42)

# Directories for storing results and images
img.dir <- "Images/"  # To store images
res.dir <- "SimData/"  # Directory for simulated data

################################################################################

# Create a 40x40 grid with a "true" structure.
d1 <- 40
d2 <- 40
v <- d1*d2
tru1 <- matrix(nrow= d1, ncol= d2)  
tru2 <- tru1


# The 'structure' in this case is simply a Gaussian density centered at the 
# center of the matrix. We also create another bright spot that is not part of 
# the structure of interest
for (r in 1:d1){
  for (c in 1:d2){
    
    tru1[r, c] <- dmvnorm(c(r, c), mean= c(20, 20), sigma= diag(c((20/3)^2, (20/3)^2)),
                          log= FALSE)
    
    tru2[r, c] <- dmvnorm(c(r, c), mean= c(30, 30), 
                          sigma= ((20/3)^2)*matrix(c(1, -0.75, -0.75, 1), nrow= 2, ncol= 2), 
                          log= FALSE)
  }
}  # End outer loop


tru1 <- (tru1 - min(tru1))/(max(tru1)-min(tru1))  # Rescale to [0,1]
tru2 <- (tru2 - min(tru2))/(max(tru2)-min(tru2))

tru <- tru1 + tru2

tru <- (tru - min(tru))/(max(tru)-min(tru))  # Rescale to [0,1]

tru1 <- tru1*(tru1 >= 0.75)  # "Slice" each image at 0.75 to create the structure
tru2 <- tru2*(tru2 >= 0.75)
tru <- tru*(tru >= 0.75)  # The target structure is tru*(tru1 ! = 0)


# Blur the image and standardize to create the first covariate image
tru.blur <- as.matrix(blur(as.im(tru), sigma= 2))
tru.blur.std <- (tru.blur - mean(tru.blur))/sd(tru.blur)  # <- 1st covariate

################################################################################

# Now create "good" and "bad" atlas labelings
# 1. "Good" label: Basically parked on top of the true structure
gd.atls <- (tru.blur.std*(tru1 != 0) >= 2)*1


# Three "Bad" atlases that are similar to each other, but all missing the mark
bd.atls1 <- tru
bd.atls1[] <- 0
bd.atls1[12:18, 22:28] <- 1 # Centered @ (20, 30)

# Make the other bad atlases "similar" by shifting them around
bd.atls2 <- bd.atls1
bd.atls3 <- bd.atls1
for (r in 1:d1){
  for (c in 1:d2){
    
    bd.atls2[r, c] <- bd.atls1[max(r-3, 1), c]
    bd.atls3[r, c] <- bd.atls1[max(r-2, 1), max(c-2, 1)]
    
  }
}

################################################################################

# Simulate intensity differences to correspond to each atlas
gd.diff <- matrix(rnorm(d1*d2), nrow= d1, ncol= d2)

# For the "bad" ones, they are 'mostly' reliable, but a have greater
# disagreement near the segmented region
bd.diff1 <- matrix(rnorm(d1*d2), nrow= d1, ncol= d2)
bd.diff1[bd.atls2 == 1] <- 1.25*bd.diff1[bd.atls2 == 1] + 2

bd.diff2 <- matrix(rnorm(d1*d2), nrow= d1, ncol= d2)
bd.diff2[bd.atls3 == 1] <- 1.25*bd.diff2[bd.atls3 == 1] - 2

bd.diff3 <- matrix(rnorm(d1*d2), nrow= d1, ncol= d2)
bd.diff3[bd.atls1 == 1] <- 1.25*bd.diff3[bd.atls1 == 1] + 2


# Blur them
gd.blur <- as.matrix(blur(as.im(gd.diff), sigma= 0.75))
bd.blur1 <- as.matrix(blur(as.im(bd.diff1), sigma= 0.75))
bd.blur2 <- as.matrix(blur(as.im(bd.diff2), sigma= 0.75))
bd.blur3 <- as.matrix(blur(as.im(bd.diff3), sigma= 0.75))


################################################################################

# Plot and save
tgt.str <- levelplot(tru*(tru1 != 0), col.regions= gray(0:32/32), xlab= "", 
                     ylab= "", 
                     scales= list(draw= F), at= seq(0, 1, length= 25), 
                     colorkey= TRUE)

cov1.plot <- levelplot(tru.blur.std, col.regions= gray(0:32/32),  
                       scales= list(draw= F), 
                       xlab= "", ylab= "", colorkey= TRUE)

gd.atls.plot <- levelplot(gd.atls, col.regions= gray(0:32/32),  
                          scales= list(draw= F), 
                   xlab= "", ylab= "", colorkey= TRUE)

bd.atlas.plot <- levelplot(bd.atls1 + bd.atls2 + bd.atls3, 
                           col.regions= gray(0:32/32),  
                  scales= list(draw= F), xlab= "", ylab= "", colorkey= TRUE)



# This creates Figure 4 in the manuscript
pdf(paste0(img.dir, "TargetAtalasBad.pdf"), height= 7, width= 28)
grid.arrange(tgt.str, cov1.plot, gd.atls.plot, bd.atlas.plot, nrow= 1)
dev.off()



dbplot1 <- levelplot(gd.blur^2, col.regions= gray(0:32/32),  
                     scales= list(draw= F), 
                     xlab= "", ylab= "", at= seq(-0.5, 5, length= 25), 
                     colorkey= TRUE)
dbplot2 <- levelplot(bd.blur1^2, col.regions= gray(0:32/32),  
                     scales= list(draw= F), 
                     xlab= "", ylab= "", at= seq(-0.5, 5, length= 25), 
                     colorkey= TRUE)
dbplot3 <- levelplot(bd.blur2^2, col.regions= gray(0:32/32),  
                     scales= list(draw= F), 
                     xlab= "", ylab= "", at= seq(-0.5, 5, length= 25), 
                     colorkey= TRUE)
dbplot4 <- levelplot(bd.blur3^2, col.regions= gray(0:32/32),  
                     scales= list(draw= F), 
                     xlab= "", ylab= "", at= seq(-0.5, 5, length= 25), 
                     colorkey= TRUE)

# This creates Supp Figure 1 in the manuscript
pdf(paste0(img.dir, "SimSqIntensityDiff.pdf"))
grid.arrange(dbplot1, dbplot2, dbplot3, dbplot4)
dev.off()

################################################################################

# Save the target, intensity covariate, atlases, and intensity differences
save("tru", "tru1", "tru.blur.std", "gd.atls", "bd.atls1", "bd.atls2", 
     "bd.atls3", "gd.blur", "bd.blur1", "bd.blur2", "bd.blur3",
     file= paste0(res.dir, "BadSegsSimData.Rdata"))
