#
# BadSeg_Images.R
#
# This produces the images (after MCMC) of the "bad but in agreement" simulation
# study in the manuscript. 
################################################################################

rm(list= ls())


require(lattice)
require(gridExtra)
require(coda)
require(tidyverse)


mcmc.out.dir <- "Results/"  # Where the MCMC output is located
img.dir <- "Images/"  # To store images

# Load the results
load(paste0(mcmc.out.dir, "BadSeg_MCMC_results.Rdata"))

# Dimensions of the image
d1 <- 40
d2 <- 40

################################################################################

# Trace plots of the regression coefficients
keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])

# This reproduces Supplementary Figure 2
pdf(paste0(img.dir, "BadSegTracePlots.pdf"), height= 7, width= 28)
par(mfrow= c(1, 4))
geweke.diag(as.mcmc(beta.MCMC[keep, 1]))
autocorr(as.mcmc(beta.MCMC[keep, 1]))
plot(beta.MCMC[keep, 1], type= "l", xlab= "Iteration", ylab= 
       expression(delta[0]))
mtext("Z = 0.230, Lag 1 Autocorr = 0.187", 3)

geweke.diag(as.mcmc(beta.MCMC[keep, 2]))
autocorr(as.mcmc(beta.MCMC[keep, 2]))
plot(beta.MCMC[keep, 2], type= "l", xlab= "Iteration", ylab= 
       expression(delta[0]))
mtext("Z = -0.421, Lag 1 Autocorr = 0.112", 3)

geweke.diag(as.mcmc(beta.MCMC[keep, 3]))
autocorr(as.mcmc(beta.MCMC[keep, 3]))
plot(beta.MCMC[keep, 3], type= "l", xlab= "Iteration", ylab= 
       expression(delta[0]))
mtext("Z = 0.314, Lag 1 Autocorr = 0.056", 3)

geweke.diag(as.mcmc(beta.MCMC[keep, 4]))
autocorr(as.mcmc(beta.MCMC[keep, 4]))
plot(beta.MCMC[keep, 1], type= "l", xlab= "Iteration", ylab= 
       expression(delta[0]))
mtext("Z = 0.371, Lag 1 Autocorr = 0.005", 3)
dev.off()



################################################################################

# Posterior means and SD maps
prb.mean <- apply(probs.MCMC[keep, ], 2, mean)
prb.se <- apply(probs.MCMC[keep, ], 2, sd)


tru.img <- levelplot(1*(tru*(tru1 != 0)), col.regions= gray(0:32/32), xlab= "", 
                     ylab= "",
                     scales= list(draw= F), at= seq(0, 1, length= 25), 
                     colorkey= TRUE)

prb.mn <- levelplot(matrix(prb.mean, nrow= d1, ncol= d2), 
                    col.regions= gray(0:32/32),
                    scales= list(draw= FALSE), at= seq(0, 1, length= 25), 
                    xlab= "", ylab= "")

prb.sd <- levelplot(matrix(prb.se, nrow= d1, ncol= d2), 
                    col.regions= gray(0:32/32),
                    scales= list(draw= FALSE), at= seq(0, 0.5, length= 25), 
                    xlab= "", ylab= "")

# This produces Figure 5 in the manuscript
pdf(paste0(img.dir, "BadSegPostResults.pdf"), height= 7, width= 21)
grid.arrange(tru.img, prb.mn, prb.sd, nrow= 1)
dev.off()


################################################################################

# Plot reliability images
# Sens./spec. fields
mean.se <- vector("list", R)
mean.sp <- vector("list", R)
se.plots <- vector("list", R)
sp.plots <- vector("list", R)
for (r in 1:R){
  
  mean.se[[r]] <- apply(pnorm(eta.SeR.MCMC[[r]][keep, ]), 2, mean)
  mean.sp[[r]] <- apply(pnorm(eta.SpR.MCMC[[r]][keep, ]), 2, mean)
  
  se.plots[[r]] <- levelplot(matrix(mean.se[[r]], nrow= d1, ncol= d2), 
                             col.regions= gray(0:32/32), 
                             scales= list(draw= FALSE),
                             xlab= "", ylab= "", at= seq(0, 1, length= 25))
  sp.plots[[r]] <- levelplot(matrix(mean.sp[[r]], nrow= d1, ncol= d2), 
                             col.regions= gray(0:32/32), 
                             scales= list(draw= FALSE),
                             xlab= "", ylab= "", at= seq(0, 1, length= 25))
}

# These two pdfs are the left and right halves of Supplementary Figure 3
pdf(paste0(img.dir, "BadSegSensImg.pdf"))
grid.arrange(se.plots[[1]], se.plots[[2]], se.plots[[3]], se.plots[[4]])
dev.off()

pdf(paste0(img.dir, "BadSegSpecImg.pdf"))
grid.arrange(sp.plots[[1]], sp.plots[[2]], sp.plots[[3]], sp.plots[[4]])
dev.off()



################################################################################

# Thresholded segmentation and results from voting procedures
tru.thresh <- 1*(tru*(tru1 != 0) != 0) # This is the truth (treated as binary)


# Thresholded BLF
bsr.thresh <- 1*(prb.mean > 0.5)


# Majority voting
maj.vot <- gd.atls + bd.atls1 + bd.atls2 + bd.atls3
maj.vot <- maj.vot >= 3
maj.vot[maj.vot == 2] <- rbinom(1, 1, 0.5)  # coin flip for ties


# Globally-weighted majority voting
gw.maj.vot1 <- gd.atls/mean(gd.blur^2) + bd.atls1/mean(bd.blur1^2) +
  bd.atls2/mean(bd.blur2^2) + bd.atls3/mean(bd.blur3^2)

gw.maj.vot0 <- (1-gd.atls)/mean(gd.blur^2) + (1-bd.atls1)/mean(bd.blur1^2) +
  (1-bd.atls2)/mean(bd.blur2^2) + (1-bd.atls3)/mean(bd.blur3^2)

gw.maj.vot <- matrix(0, nrow= d1, ncol= d2)
for (i in 1:d1){
  for (j in 1:d2){
    
    gw.maj.vot[i,j] <- ifelse(gw.maj.vot1[i,j] > gw.maj.vot0[i,j], 1, 0) 
    
  }
}

gw.maj.vot[gw.maj.vot1 == gw.maj.vot0] <- rbinom(1, 1, 0.5)


# Locally-weighted majority voting
lw.maj.vot1 <- gd.atls/gd.blur^2 + bd.atls1/bd.blur1^2 +
  bd.atls2/bd.blur2^2 + bd.atls3/bd.blur3^2
lw.maj.vot0 <- (1-gd.atls)/gd.blur^2 + (1-bd.atls1)/bd.blur1^2 +
  (1-bd.atls2)/bd.blur2^2 + (1-bd.atls3)/bd.blur3^2

lw.maj.vot <- matrix(0, nrow= d1, ncol= d2)
for (i in 1:d1){
  for (j in 1:d2){
    
    lw.maj.vot[i,j] <- ifelse(lw.maj.vot1[i,j] > lw.maj.vot0[i,j], 1, 0) 
    
  }
}

lw.maj.vot[lw.maj.vot1 == lw.maj.vot0] <- rbinom(1, 1, 0.5)


# BLF Dice
tab.bsr <- table(c(tru.thresh), c(bsr.thresh))
denom.bsr <- 2*tab.bsr[2,2] + tab.bsr[1,2] + tab.bsr[2,1]
dice.bsr <- (2*tab.bsr[2,2])/denom.bsr
print(dice.bsr)  # 0.938


# MV Dice
tab.mv <- table(c(tru.thresh), maj.vot)
denom.mv <- 2*tab.mv[2,2] + tab.mv[1,2] + tab.mv[2,1]
dice.mv <- (2*tab.mv[2,2])/denom.mv
print(dice.mv)  # 0.081


# GW Maj. Voting Dice
tab.gw.mv <- table(c(tru.thresh), gw.maj.vot)
denom.gw.mv <- 2*tab.gw.mv[2,2] + tab.gw.mv[1,2] + tab.gw.mv[2,1]
dice.gw.mv <- (2*tab.gw.mv[2,2])/denom.gw.mv
print(dice.gw.mv)  # 0.225


# LW Maj. Voting Dice
tab.lw.mv <- table(c(tru.thresh), lw.maj.vot)
denom.lw.mv <- 2*tab.lw.mv[2,2] + tab.lw.mv[1,2] + tab.lw.mv[2,1]
dice.lw.mv <- (2*tab.lw.mv[2,2])/denom.lw.mv
print(dice.lw.mv)  # 0.477


###############################

# Plot the 4 segmentations
bin.prb.plot <- levelplot(matrix(1*(prb.mean > 0.5), nrow= d1, ncol= d2), 
                          col.regions= gray(0:32/32),
                          scales= list(draw= FALSE), colorkey= FALSE, xlab= "", 
                          ylab= "", main= "Threshold PPM (D = 0.938)")

mv.plot <- levelplot(maj.vot, col.regions= gray(0:32/32),  
                     scales= list(draw= F), 
                     xlab= "", ylab= "", colorkey= FALSE, 
                     main= "Maj. Vote (D = 0.081)")

gw.mv.plot <- levelplot(gw.maj.vot, col.regions= gray(0:32/32),  
                        scales= list(draw= F), 
                        xlab= "", ylab= "", colorkey= FALSE, 
                        main= "GW Maj. Vote (D = 0.225)")

lw.mv.plot <- levelplot(lw.maj.vot, col.regions= gray(0:32/32),  
                        scales= list(draw= F), 
                        xlab= "", ylab= "", colorkey= FALSE, 
                        main= "LW Maj. Vote (D = 0.477)")

# This produces the left panel of Figure 6 in the manuscript
pdf(paste0(img.dir, "BadSegFourSegs.pdf"))
grid.arrange(bin.prb.plot, mv.plot, gw.mv.plot, lw.mv.plot)
dev.off()


############################

# Volume Estimates
vol.ests.rb <- apply(probs.MCMC[keep, ], 1, sum)  # BLF vol. distribution



# This produces the right panel of Figure 6 in the manuscript. The commented
# out part will produce a black and white version. The uncommented code 
# produces a color figure.
# 
# mv.vol <- sum(maj.vot)  # maj. vote volume estimate
# gw.mv.vol <- sum(gw.maj.vot)  # GW maj. vote
# lw.mv.vol <- sum(lw.maj.vot)  # LW maj. vote
#
# pdf(paste0(img.dir, "BadSegVolEstimates.pdf"))
# plot(density(vol.ests.rb), lwd= 2, lty= 1, main= "", xlab= "Volume", 
#      ylab= "Density", xlim= c(20, 120), ylim= c(0, 0.1))
# abline(v= sum(tru1 != 0), lwd= 2)  # Binary version
# abline(v= mv.vol, lwd= 2, lty= 2)
# abline(v= gw.mv.vol, lwd= 2, lty= 3)
# abline(v= lw.mv.vol, lwd= 2, lty= 4)
# legend("topright", lwd= 2, lty= 1:4, legend= c("(Binary) Truth", "Maj. Vote", 
#                                                "GW Maj. Vote", "LW Maj. Vote"),
#        bty= "n")
# dev.off()


man_vol <- sum(tru1 != 0)
lwmv_vol <- sum(lw.maj.vot)
gwmv_vol <- sum(gw.maj.vot)
mv_vol <- sum(maj.vot)

blf_vol_dens <- data.frame(Vol= vol.ests.rb)

# Stack the other segmentations
seg_type <- c("Manual", "LW Maj. Vote", "GW Maj. Vote", "Maj. Vote")
seg_vol <- c(man_vol, lwmv_vol, gwmv_vol, mv_vol)
segs_df <- data.frame(Volume= seg_vol, Segmentation= seg_type)

pdf(paste0(img.dir, "BadSegVolEstimates_pretty.pdf"))
ggplot(blf_vol_dens, aes(x = Vol)) + 
  xlim(20, 120) +
  xlab(bquote("Volume"~"(mm"^3~")")) +
  ylab("Density") +
  geom_density(fill= "red", alpha= 0.3) +
  geom_vline(aes(xintercept = Volume, color= Segmentation, 
                 linetype= Segmentation), size= 0.75,
             data= segs_df)
dev.off()