#
# GoodSeg_SDL_MCMC.R
#
# Code for repdroducing the "good segmentation" MCMC results with the SDL
# covariate.
#
################################################################################

rm(list= ls())

require(R.matlab)
require(Matrix)

set.seed(314)


# Addresses for reading and writing
sim.dat.dir <- "SimData\\"  # Where the simulated data are stored
mcmc.out.dir <- "Results\\"  # Where to store the MCMC output

# Load the MCMC functions
source("Sim_MCMC_Fns.R")

# Iteration and thinning parameters for the MCMC
iter <- 100000
thin <- 25

# Load the simulated data
load(paste0(sim.dat.dir, "GoodSegsSimData.Rdata"))

#-------------------------------------------------------------------------------

# Model setup
segmentations <- list(trans.seg, rot.seg, dil.seg, con.seg)

R <- 4  # R = total number of available atlases.
M <- 1    # Number of coefficients for spatially varying sensitivities and 
  # specificities (same for both) including intercept
P <- 2    # Number of coefficients (including possibly and intercept)

P1 <- dim(segmentations[[1]])[1]
P2 <- dim(segmentations[[1]])[2]
V <- P1*P2  # number of voxels in one slice
Vid <- array(1:V, c(P1,P2)) # Identifies voxels  
Y <- matrix(-99, V, R)     # Outcome based on Segmentation R

X <- matrix(nrow=V, ncol=P) # Design matrix for true status
Dj <- rep(-99, V)  # The voxel-specific "weights" for the CAR model
Nj <- matrix(-99, nrow=V, ncol=8) # This will list the neighbors for each voxel.


id1 <- rep(-99, V)  # For labeling the "colors" in the Markov graph based on the 
# assumed dependence structure.
id2 <- rep(-99, V)
id3 <- rep(-99, V)
id4 <- rep(-99, V)

c1 <- 1  # Counters to keep track of how many we have of each color.
c2 <- 1
c3 <- 1
c4 <- 1




# Run MATLAB to get the SDLs and store them appropriately
Matlab$startServer()
matlab <- Matlab()  # Creates the connection 
isOpen <- open(matlab)


# Pass binary images to MATLAB
setVariable(matlab, trans_seg= trans.seg, dil_seg= dil.seg, con_seg= con.seg,
            rot_seg= rot.seg)  

# Call the bwdist function to find the distance label images. Note that you need
# the image processing toolbox in MATLAB for this to work. The distances are 
# signed so that negative values indicate 'inside' the labeled structure,
# positive values indicate 'outside', and zeros indicate edge points.
mat.com <- "trans_sDist= bwdist(trans_seg) - bwdist(1-trans_seg);
dil_sDist= bwdist(dil_seg) - bwdist(1-dil_seg);
con_sDist= bwdist(con_seg) - bwdist(1-con_seg);
rot_sDist= bwdist(rot_seg) - bwdist(1-rot_seg);"

evaluate(matlab, mat.com)  # The runs the code in MATLAB


# Return the variables and close MATLAB
trans.sDist <- getVariable(matlab, "trans_sDist")
dil.sDist <- getVariable(matlab, "dil_sDist")
con.sDist <- getVariable(matlab, "con_sDist")
rot.sDist <- getVariable(matlab, "rot_sDist")

close(matlab)


# Extract the actual values (1st element of the list)
trans.sDist <- trans.sDist[[1]]
dil.sDist <- dil.sDist[[1]]
con.sDist <- con.sDist[[1]]
rot.sDist <- rot.sDist[[1]]

sum.sDist <- trans.sDist + dil.sDist + con.sDist + rot.sDist



# Set up neighborhood structure, colorings for the chromatic sampler, the 
# design matrix, etc.
for(p1 in 1:P1){
  for(p2 in 1:P2){
    
    # Create the neighborhood matrix
    l1 = max(c(1, p1-1))  # Lower bound along dim 1 (rows)
    l2 = max(c(1, p2-1))  # cols
    u1 = min(c(P1, p1+1))  # Upper bound along dim 1 (rows)
    u2 = min(c(P2, p2+1))  # cols
    
    neighbors = as.vector(Vid[l1:u1, l2:u2])  # Vectorize the labels 
      # corresponding to the point of interest.
    
    neighbors = neighbors[neighbors != Vid[p1, p2]]  # Can't be a neighbor with
      # yourself!
    
    wjv = length(neighbors)  # The w_j weights = number of neighbors for 
      # voxel j
    
    Dj[Vid[p1, p2]] = wjv  # Store it in weight matrix
    Nj[Vid[p1, p2], 1:wjv] = neighbors  
    
    # Systematically filling in the colors based on row/col combination.
    if(p1%%2!=0 & p2%%2!=0){ 
      
      id1[c1]<-Vid[p1,p2]
      c1<-c1+1
      
    }  # Odd rows and odd cols
    if(p1%%2!=0 & p2%%2==0){ 
      
      id2[c2]<-Vid[p1,p2]
      c2<-c2+1
      
    }  # Odd rows and even cols
    if(p1%%2==0 & p2%%2!=0){ 
      
      id3[c3]<-Vid[p1,p2]
      c3<-c3+1
      
    }  # Even rows and odd cols
    if(p1%%2==0 & p2%%2==0){ 
      
      id4[c4]<-Vid[p1,p2]
      c4<-c4+1
      
    }  # Even rows and even cols
    
    # Here is where we are storing the covariate info in the design matrix.
    X[Vid[p1, p2], 1] <- 1
    X[Vid[p1, p2], 2] <- sum.sDist[p1, p2]


    
    for(r in 1:R){
      
      Y[Vid[p1,p2],r] <- segmentations[[r]][p1,p2]  # Simply creating the 
        # array of observed labels
      
    }  # End for    
  }  # End for
}  # End for



id1 <- id1[id1 != -99]  # Collapse the color vectors to only contain "real" 
  # voxel labels
id2 <- id2[id2 != -99]
id3 <- id3[id3 != -99]
id4 <- id4[id4 != -99]

n.id1 <- length(id1)  # Number of voxels corresponding to each color. In the 
n.id2 <- length(id2)    # 50x50 case, we have 625 ea.
n.id3 <- length(id3)
n.id4 <- length(id4)



# Creating the sparse matrices to store the Laplacian. This is done more 
# efficiently in the ADNI example by pre-allocating the memory. It doesn't 
# matter much here.
i = c()
j = c()
for(v in 1:V){
  
  i = c(i, rep(v, Dj[v]))  # Building the index set to know where to put the 1's
    # in the sparse matrix. 
  j = c(j, Nj[v, 1:Dj[v]])
  
}  # End for


# Sparse matrix representation
rho <- 0.995
Wj <- sparseMatrix(i, j, x=1)  
Dj <- sparseMatrix(1:V, 1:V, x=Dj)
DjWj <- Dj-rho*Wj   # making the model proper by fixing the propriety parameter!
I <- sparseMatrix(1:V,1:V,x=1)
Dj <- diag(Dj) 



###################

# Prior specification

alam.SeR <- vector("list", R)
alam.SeR[1:R] <- 1 # hyperparameters for the prior on lambda sens. model. A is
  # the shape, b is the rate

blam.SeR <- vector("list", R)
blam.SeR[1:R] <- 1

alam.SpR <- vector("list", R)
alam.SpR[1:R] <- 1 # hyperparameters for the prior on lambda spec. model.

blam.SpR <- vector("list", R)
blam.SpR[1:R] <- 1


# Specified mean of spatial process
mu.SeR <- lapply(1:R, function(x) rep(0, V))   # Change mean of spatial process 
  # here, if desired.
mu.SpR <- lapply(1:R, function(x) rep(0, V))



# BCJ prior specification. This is where we can indirectly induce a prior on
# the regression coefficient(s) via CMP elicitation. 
x.add <- rbind(c(1, -3), c(1, 1))
p.add <- c(0.9, 0.1)  # 90% chance a voxel with a SDL = -3 is in the structure
  # 10% chance a voxel with SDL = 2 is in the structure
n.add <- c(50, 50)  
y.add <- n.add*p.add  # "additional" data for DA step (= CMP under logistic
  # link)


#-------------------------------------------------------------------------------

# Initialize the MCMC

# Initialize eta (i.e., the spatial processes representing the linear predictor
# part of the probit link model on Se and Sp)
eta.SeR <- lapply(1:R, function(x) rep(1, V))  
eta.SpR <- lapply(1:R, function(x) rep(1, V))


# Initialize precisions for sens/spec processes
lamj.SeR <- vector("list", R)
lamj.SeR[1:R] <- 1
lamj.SpR <- vector("list", R)
lamj.SpR[1:R] <- 1

Ts <- rbinom(V, 1, apply(Y, 1, mean))   # Use these beta values as the initial 
  # state for the Gamerman MH proposal
beta.init <- coef(glm(c(Ts, y.add/n.add) ~ rbind(X, x.add)-1, 
                      family= "binomial",
                      weights= c(rep(1, times= length(Ts)), n.add) ))
  # Ignore the warning since the data are "hypothetical"



# Create some storage space
Ts.MCMC <- matrix(-99, nrow=iter/thin, ncol=V) 
probs.MCMC <- matrix(-99, nrow= iter/thin, ncol= v)

eta.SeR.MCMC <- lapply(1:R, function(x) return(matrix(-99, 
                                                      nrow=iter/thin, ncol=V)))
eta.SpR.MCMC <- lapply(1:R, function(x) return(matrix(-99, 
                                                      nrow=iter/thin, ncol=V)))

lamj.SeR.MCMC <- lapply(1:R, function(x) return(matrix(-99, 
                                                       nrow=iter/thin, ncol=1)))
lamj.SpR.MCMC <- lapply(1:R, function(x) return(matrix(-99, 
                                                       nrow=iter/thin, ncol=1)))

beta.MCMC <- matrix(-99, nrow=iter/thin, ncol=(P))

#-------------------------------------------------------------------------------

# Run the MCMC
beta <-  beta.init

system.time(
for(i in 1:iter){
  
  ######################################################
  # Sampling beta with a standard N(0,t2*I)
  
  # Auxiliary functions are defined in source script above
  beta <- MH.logistic.reg(b0=beta, X=X, y=Ts, Y.add=y.add, n.add=n.add,
                          X.add=x.add)
  Xb <- X%*%beta
  
  #####################################################
  # Sampling latent Zj
  getZj <- sampleZj(eta.SeR, eta.SpR, Ts, Y, R, V)
  Zj.SeR <- getZj$Zj.SeR 
  Zj.SpR <- getZj$Zj.SpR 
  
  
  ######################################################
  # Sampling the spatial random effects phi  
  getEta <- sampleEta(lamj.SeR, lamj.SpR, mu.SeR, mu.SpR, Zj.SeR, Zj.SpR, Dj, 
                      Wj, 
                      Ts, R, rho, eta.SeR, eta.SpR, id1, id2, id3, id4, n.id1,
                      n.id2, n.id3, n.id4)
  eta.SeR <- getEta$eta.SeR
  eta.SpR <- getEta$eta.SpR
  
  ######################################################
  # Sampling lamj
  getLamj <- sampleLamj(alam.SeR, alam.SpR, blam.SeR, blam.SpR, eta.SeR,
                        eta.SpR, 
                        mu.SeR, mu.SpR, DjWj, R, V)
  lamj.SeR <- getLamj$lamj.SeR
  lamj.SpR <- getLamj$lamj.SpR
  
  ######################################################
  # Sampling latent T's  
  bin.out <- sampleTs(Xb, eta.SeR, eta.SpR, Y, V, R)
  
  Ts <- bin.out[[1]]
  prb <- bin.out[[2]]
  
  
  ######################################################
  # Store the sample   
  if( i %% thin == 0 ){
    
    j <- i/thin
    Ts.MCMC[j, ] <- Ts
    probs.MCMC[j, ] <- prb
    beta.MCMC[j, ] <- beta
    
    for(r in 1:R){
      
      eta.SeR.MCMC[[r]][j,] <-eta.SeR[[r]]
      lamj.SeR.MCMC[[r]][j,]<-lamj.SeR[[r]]
      
      eta.SpR.MCMC[[r]][j,] <-eta.SpR[[r]]
      lamj.SpR.MCMC[[r]][j,]<-lamj.SpR[[r]]
      
    }  # End for
    
  }  # End if
  
  # Progress tracker
  if (i %% 100 == 0){
    print(i)
  }
  

}  # End MCMC loop
)  # End system time   user     system  elapsed 
   #                  2605.82    5.39   2613.29 

#-------------------------------------------------------------------------------


# Save the output
save.image(paste0(mcmc.out.dir, "GoodSeg_SDL_MCMC_results.Rdata"))
