# @file registration.R
# @author Kristin Linn
# @date 2-10-16
# @brief Fourth step in the ADNI data preprocessing pipeline.

library (parallel)
library (methods)
library(ANTsR)
library(extrantsr)

# Run subjects in parallel
args = commandArgs(trailingOnly=TRUE)
jobInd = as.numeric(args[1])

# Directories
rootdir = '/project/hippocampus_segmentation'
atlasdir = file.path(rootdir, 'atlases')
labeldir = file.path(rootdir, 'labels')
opdir = file.path(rootdir, 'output')

# NOTE: We will use the $LSB_JOBINDEX atlas as our target
atlases = list.files(atlasdir)
system(paste0('mkdir ', file.path(opdir, atlases[jobInd])))
targetdir = file.path(opdir, atlases[jobInd])
fixedFile = file.path(atlasdir, atlases[jobInd])
# Read in the target T1
fixedT1 = antsImageRead(file.path(fixedFile, 'PAD_N4_T1_brain.nii.gz'), 3)
  
# Parallelize over atlases
reg = function(i){
    # Don't register target to itself
    if(i==jobInd){
        return(NULL)
    }
    atlasFile = file.path(atlasdir, atlases[i])
    T1 <- antsImageRead(file.path(atlasFile, 'PAD_N4_T1_brain.nii.gz'), 3)
    # Register atlas T1 to target T1
    atlas_to_T1 = antsRegistration(fixed=fixedT1, 
        moving=T1, typeofTransform = c("SyN"),
        outprefix=file.path(targetdir, paste0(atlases[i], '_REGT1_N4_ATL')))
    REGT1_N4_ATL = antsApplyTransforms(fixed=fixedT1, 
        moving=T1, 
        transformlist = atlas_to_T1$fwdtransforms, 
        interpolator=c("WelchWindowedSinc"))
    antsImageWrite(REGT1_N4_ATL, file.path(targetdir, paste0(atlases[i], '_REGT1_N4_ATL.nii.gz')))
    # Apply transformation to the segmentations
    leftFile = file.path(atlasdir, atlases[i], paste0(atlases[i], '_L_PAD.nii.gz'))
    rightFile = file.path(atlasdir, atlases[i], paste0(atlases[i], '_R_PAD.nii.gz'))
    LEFT = as.array(antsImageRead(leftFile, 3))
    LEFT = as.antsImage(LEFT, reference=T1)
    RIGHT = as.array(antsImageRead(rightFile, 3))
    RIGHT = as.antsImage(RIGHT, reference=T1)
    TRANS_LEFT = antsApplyTransforms(fixed=fixedT1, 
        moving=LEFT, 
        transformlist = atlas_to_T1$fwdtransforms, 
        interpolator=c("genericLabel"))    
    TRANS_RIGHT = antsApplyTransforms(fixed=fixedT1, 
        moving=RIGHT, 
        transformlist = atlas_to_T1$fwdtransforms, 
        interpolator=c("genericLabel"))
    antsImageWrite(TRANS_LEFT, file.path(targetdir, paste0(atlases[i], '_REGT1_LEFT.nii.gz')))
    antsImageWrite(TRANS_RIGHT, file.path(targetdir, paste0(atlases[i], '_REGT1_RIGHT.nii.gz')))
}

op = mclapply(1:12, reg, mc.cores=4)



####################################
if (FALSE){

> sessionInfo()
R version 3.2.5 (2016-04-14)
Platform: x86_64-pc-linux-gnu (64-bit)
Running under: CentOS Linux 7 (Core)

locale:
 [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
 [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
 [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
 [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
 [9] LC_ADDRESS=C               LC_TELEPHONE=C            
[11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       

attached base packages:
[1] parallel  stats     graphics  grDevices utils     datasets  methods  
[8] base     

other attached packages:
[1] extrantsr_2.17 ANTsR_0.4.0   

loaded via a namespace (and not attached):
 [1] Rcpp_0.12.14            plyr_1.8.4              later_0.7.5            
 [4] pillar_1.4.3            R.matlab_3.6.2          R.methodsS3_1.7.1      
 [7] bitops_1.0-6            R.utils_2.7.0           iterators_1.0.10       
[10] tools_3.2.5             digest_0.6.18           yaImpute_1.0-30        
[13] jsonlite_1.5            WhiteStripe_2.2.2       tibble_3.0.3           
[16] lifecycle_0.2.0         nlme_3.1-131            lattice_0.20-35        
[19] mgcv_1.8-24             pkgconfig_2.0.1         rlang_0.4.7            
[22] Matrix_1.2-15           shiny_1.0.3             RNifti_0.7.1           
[25] oro.nifti_0.9.1         crosstalk_1.0.0         knitr_1.20             
[28] stringr_1.3.1           htmlwidgets_1.3         vctrs_0.3.2            
[31] hms_0.4.2               webshot_0.5.1           manipulateWidget_0.10.0
[34] grid_3.2.5              neurobase_1.19.1        R6_2.2.2               
[37] hash_2.2.6              fslr_2.22.0             readr_1.3.1            
[40] magrittr_1.5            promises_1.0.1          htmltools_0.3.6        
[43] matrixStats_0.52.2      ellipsis_0.3.0          splines_3.2.5          
[46] abind_1.4-5             assertthat_0.2.0        xtable_1.8-3           
[49] mime_0.6                httpuv_1.4.5            miniUI_0.1.1.1         
[52] stringi_1.2.4           neuroim_0.0.6           crayon_1.3.4           
[55] R.oo_1.22.0    


}



