# @file 8_subset_images.R
# @author Kristin Linn
# @brief Extract slice numbers that contain hippocampus and restrict images to those

# Required packages
library(parallel)
library(methods)
library(neurobase)

# Run subjects in parallel
args = commandArgs(trailingOnly=TRUE)
jobInd = as.numeric(args[1])

rootDir = '/project/hippocampus_segmentation'
imgDir = file.path(rootDir, 'atlases')
segDir = file.path(rootDir, 'output')
segIds = c('0016', '0067', '0088', '0172', '0221', '0413', '0419', '0525', '0592', '0754', '0784', '1263')

leftNames = lapply(segIds, function(x) paste0(x, '_L_PAD.nii.gz'))
rightNames = lapply(segIds, function(x) paste0(x, '_R_PAD.nii.gz'))
t1Names = 'WS_PAD_N4_T1.nii.gz'

# Parallelize over targets
run = function(i){
	target = segIds[i]
	# target image
	T1 = readnii(file.path(imgDir, target, 'PAD_N4_T1_brain.nii.gz'))
	normT1 = readnii(file.path(imgDir, target, 'WS_PAD_N4_T1.nii.gz'))
	# true segmentation
	trueLeft = readnii(file.path(imgDir, target, leftNames[[i]]))
	trueRight = readnii(file.path(imgDir, target, rightNames[[i]]))
	TRUE_SEG = trueLeft + trueRight
	# atlas segmentations in target space
	aNames = segIds[-i]
	leftSegFiles = lapply(1:11, function(x) file.path(segDir, target, paste0(aNames[x], '_REGT1_LEFT.nii.gz')))
	rightSegFiles = lapply(1:11, function(x) file.path(segDir, target, paste0(aNames[x], '_REGT1_RIGHT.nii.gz')))
	LEFT_SEGS = lapply(leftSegFiles, readnii)
	RIGHT_SEGS = lapply(rightSegFiles, readnii)
	SEGS = lapply(1:11, function(x) LEFT_SEGS[[x]]+RIGHT_SEGS[[x]])
	lapply(1:11, function(x) writenii(SEGS[[x]], filename=file.path(segDir, target, paste0(aNames[x], '_REGT1_BOTH.nii.gz'))))
    # JLF segmentations
    jlfFile = file.path(rootDir, 'jlf', paste0(target, '_jlf_segmentation.nii.gz'))
    JLF = readnii(jlfFile)
    # JLF CN to AD or AD to CN
    jlfFileG = file.path(rootDir, 'jlf_by_group/jlf_3D/jlf_cn_to_ad', paste0(target, '_jlf_segmentation.nii.gz'))
	if (file.exists(jlfFileG)){
	    JLFG = readnii(jlfFile)
		regdir = 'cn_to_ad'
	} else{
	    jlfFileG = file.path(rootDir, 'jlf_by_group/jlf_3D/jlf_ad_to_cn', paste0(target, '_jlf_segmentation.nii.gz'))
	    JLFG = readnii(jlfFile)
		regdir = 'ad_to_cn'
	}
	# Atlas T1 images
	atlasT1Files = lapply(1:11, function(x) file.path(segDir, target, paste0('WS_', aNames[x], '_REGT1_N4_ATL.nii.gz')))
	ATLASES = lapply(atlasT1Files, readnii)
	imgSimilarity = lapply(ATLASES, function(x) sqrt((x - normT1)^2))
    # Fix max
	for(j in 1:length(imgSimilarity)){
		imgSimilarity[[j]]@cal_max = max(imgSimilarity[[j]])
	}
    # Add up all the segmentations
    addSegs = Reduce ('+', SEGS)
    addSegs = addSegs + TRUE_SEG
    # Create bounding box around segmentations to reduce size
	brainX = which (apply(addSegs, 1, sum)!=0)
	brainY = which (apply(addSegs, 2, sum)!=0)
	brainZ = which (apply(addSegs, 3, sum)!=0)
    # Target slices
	sliceT1 = normT1[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)]
    sliceT1 = as.nifti(sliceT1)
    # Target manual segmentation is our "gold standard"
	trueMask = TRUE_SEG[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)]
    trueMask = as.nifti(trueMask)
	sliceDir = file.path(segDir, target, 'slices3D')
	system(paste0('mkdir ', sliceDir))
	# Target tissue segmentation from FAST
	tsFile = file.path(segDir, target, 'FAST_T1.nii.gz')
	TISSEG = readnii(tsFile)
	TISSEG = TISSEG[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)]
    TISSEG = as.nifti(TISSEG)
    # Write out data 
	writenii(trueMask, filename=file.path(sliceDir, paste0(target, '_true_segmentation')))
	writenii(sliceT1, filename=file.path(sliceDir, paste0(target, '_target_slice')))
	writenii(TISSEG, filename=file.path(sliceDir, paste0(target, '_tissue_segmentation')))
    # JLF
    jlfSeg = JLF[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)]
    jlfSeg = as.nifti(jlfSeg)
    writenii(jlfSeg, filename=file.path(sliceDir, paste0(target, '_jlf_segmentation')))
    jlfSegG = JLFG[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)]
    jlfSegG = as.nifti(jlfSegG)
    writenii(jlfSegG, filename=file.path(sliceDir, paste0(target, '_jlf_segmentation_', regdir)))
    # Atlas segmentations
	sliceSegs = lapply(SEGS, function(x) x[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)])
	lapply(1:11, function(x) writenii(as.nifti(sliceSegs[[x]]), filename=file.path(sliceDir, paste0(aNames[x], '_atlas_segmentation'))))
    # Atlases
	sliceAtl = lapply(ATLASES, function(x) x[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)])
	lapply(1:11, function(x) writenii(as.nifti(sliceAtl[[x]]), filename=file.path(sliceDir, paste0(aNames[x], '_atlas_t1'))))
    # Intensity similarity covariates
	sliceSims = lapply(imgSimilarity, function(x) as.nifti((x[min(brainX):max(brainX),min(brainY):max(brainY),min(brainZ):max(brainZ)])))
	lapply(1:11, function(x) writenii(sliceSims[[x]], filename=file.path(sliceDir, paste0(aNames[x], '_similarity'))))
}

run (jobInd)


####################################
if (FALSE){

> sessionInfo()
R version 3.2.5 (2016-04-14)
Platform: x86_64-pc-linux-gnu (64-bit)
Running under: CentOS Linux 7 (Core)

locale:
 [1] LC_CTYPE=en_US.UTF-8       LC_NUMERIC=C              
 [3] LC_TIME=en_US.UTF-8        LC_COLLATE=en_US.UTF-8    
 [5] LC_MONETARY=en_US.UTF-8    LC_MESSAGES=en_US.UTF-8   
 [7] LC_PAPER=en_US.UTF-8       LC_NAME=C                 
 [9] LC_ADDRESS=C               LC_TELEPHONE=C            
[11] LC_MEASUREMENT=en_US.UTF-8 LC_IDENTIFICATION=C       

attached base packages:
[1] parallel  stats     graphics  grDevices utils     datasets  methods  
[8] base     

other attached packages:
[1] neurobase_1.19.1 oro.nifti_0.9.1 

loaded via a namespace (and not attached):
 [1] tools_3.2.5        abind_1.4-5        Rcpp_0.12.14       R.methodsS3_1.7.1 
 [5] splines_3.2.5      RNifti_0.7.1       R.utils_2.7.0      matrixStats_0.52.2
 [9] bitops_1.0-6       R.oo_1.22.0  
	
}

