#
# Sim_MCMC_Fns.R
#
# The MCMC auxiliary functions for the simulated examples in the 
# manuscript.
#
#-------------------------------------------------------------------------------

require(mnormt)
require(mvtnorm)
require(msm)

mt_ct.BCJ<-function(b, y, X, Y.add, n.add, X.add){
  ############################################
  # Finds the Mean and Covariance of proposal density according to Gammerman 
  # (1997)
  #
  # b= coefficient vector
  # y= "observed" data
  # X= "observed" design matrix
  # Y.add= "additional" reponses
  # n.add= weights (= no. trials) associated with Y.add
  # X.add= design matrix associated with "additional" responses
  
  P <- length(b)
  xb <- X%*%b
  xb.add <- X.add%*%b
  
  p <- exp(xb)/(1+exp(xb))
  p.add <- exp(xb.add)/(1+exp(xb.add))
  
  W <- p*(1-p)
  W.add <- n.add*p.add*(1-p.add)
  
  Yb <- xb*(p*(1-p))+(y-p)
  Yb.add <- xb.add*(n.add*p.add*(1-p.add)) + (Y.add-n.add*p.add)
  
  X.c <- rbind(X,X.add)
  W <- as.vector(c(W,W.add))
  Yb <- c(Yb,Yb.add)
  
  XWX <- t(X.c*W)%*%X.c
  
  C <- solve(XWX)  # NOTE: You better make sure X isn't large here!
  # C <- (C+t(C))/2  # Is C not symmetric???
  M <- C%*%(t(X.c)%*%(Yb))
  
  # Log target density
  L <- sum(y*(xb)-log(1+exp(xb))) + sum(Y.add*(xb.add)-n.add*log(1+exp(xb.add)))
  
  return(list("M"= M, "C"= C, "L"= L))
  
}  # End fn



MH.logistic.reg <- function(b0, X, y, Y.add, n.add, X.add){
  ############################################
  # MH update for a logistic link following the Gamerman algorithm
  # using the mean and cov created in mt_ct.u 
  #
  # b0 = current state of the coefficient vector
  # X = observed design matrix
  # y = observed (binary) responses
  # Y.add = augmenting (binomial) responses
  # n.add = weights (trials) associated with the augmenting responses
  # X.add = design matrix associated with the augmenting responses
  
  # This just gets the mean (M), covariance (C), and the log target density
  res <- mt_ct.BCJ(b=b0, y=y, X=X, Y.add=Y.add, n.add=n.add, X.add=X.add)
  Mb0 <- res$M
  Cb0 <- res$C
  Lb0 <- res$L
  
  
  b.star <- as.vector(rmnorm(1,Mb0,Cb0))
  qb.star <- dmvnorm(b.star, as.vector(Mb0), Cb0, log=TRUE)  # log propsal 
    # density
  
  res <- mt_ct.BCJ(b=b.star, y=y, X=X, Y.add=Y.add, n.add=n.add, X.add=X.add)
  Mb.star <- res$M
  Cb.star <- res$C
  Lb.star <- res$L
  

  qb0 <- dmvnorm(b0, as.vector(Mb.star), Cb.star, log=TRUE)
  
  ratio.pi <- exp(Lb.star-Lb0+qb0-qb.star)
  
  alpha <- min(1,ratio.pi)
  accept <- rbinom(1,1,alpha)
  if(accept == 1){
    
    b1 <- b.star
  }
  
  else {
    
    b1 <- b0
    
  }
  
  return("b1"= b1)
  
}  # End fn



sampleZj <- function(eta.SeR, eta.SpR, Ts, Y, R, V){
  #####################################################
  # Sampling latent Zj that arise from the Albert / Chib truncated normal 
  # sampling trick.
  # 
  # eta.SeR = sensitivity spatial field (= phi_r in the paper)
  # eta.SpR = specificity spatial field (= eta_r in the papar)
  # Ts = current vector of true inclusion indicators
  # Y = array of observed atlas segmentations
  # R = number of atlas segmentations
  # V = total number of voxels
  
  Zj.SeR<-lapply(1:R, function(x) rep(-99, sum(Ts)))  # Create a list, one 
    # element for each atlas, each with length = # included voxels
  Zj.SpR<-lapply(1:R, function(x) rep(-99, V-sum(Ts)))
  
  for(r in 1:R){
    
    l.se <- rep(-99,sum(Ts))  # lower/upper truncations for the sens. vars.
    u.se <- rep(-99,sum(Ts))
    l.sp <- rep(-99,(V-sum(Ts)))  # lower/upper truncations fo the spec. vars.
    u.sp <- rep(-99,(V-sum(Ts)))
    
    # Update latent sens. variables first.
    # Truncation range for Y = 0 = (-infty, 0)
    l.se[Y[Ts==1,r] == 0] <- -Inf
    u.se[Y[Ts==1,r] == 0] <- 0
    
    # Truncation range for Y = 1 = (0, infty)
    l.se[Y[Ts==1,r] == 1] <- 0
    u.se[Y[Ts==1,r] == 1] <- Inf
    
    # Sensitivites <-> phi in the paper
    Zj.SeR[[r]]<-rtnorm(sum(Ts), eta.SeR[[r]][Ts==1], 1, lower=l.se, upper=u.se)
    
    
    # Update latent spec. variable next. The truncation pattern is the opposite
    # of the sens.
    l.sp[Y[Ts==0,r] == 0] <- 0
    u.sp[Y[Ts==0,r] == 0] <- Inf
    
    l.sp[Y[Ts==0,r] == 1] <- -Inf
    u.sp[Y[Ts==0,r] == 1] <- 0
    
    # Specificities <-> eta in the paper
    Zj.SpR[[r]]<-rtnorm((V-sum(Ts)), eta.SpR[[r]][Ts==0], 1, lower=l.sp, 
                        upper=u.sp)
    
  }
  
  return(list('Zj.SeR'=Zj.SeR, 'Zj.SpR'=Zj.SpR))
  
}  # End fn



FS.GMRF <- function(Z, rho, D, W, phi, mu, lam, id1, id2, id3, id4, n.id1, 
                    n.id2, n.id3, n.id4, incls){
  ############################################################
  # Support function that updates the GMRFs via chromatic sampling.
  #
  # Z = Vector of latent trnc. Gaussians needed to induce conjugacy in the 
  #   Albert/Chib trick
  # rho = propriety parameter for the CAR model
  # W = incidence matrix for the CAR model
  # phi = current state of the field (updated in subsequent subsets according
  #   to the coloring of the Markov graph)
  # mu = mean of the GMRF
  # lam = precision of the GMRF
  # id1, ..., id4 = index sets indicating the groupings according to color
  # n.id1, ..., n.id4 = lengths of id1, ..., id4 (= numbers of elements assigned
  #   to each color)
  # incls = logical vector indicating either inclusion (T = 1) or exclusion
  #   (T = 0) for the GMRF updates. See the conditional distributions dervied
  #   in the manuscript.
  
  Yt <- rep(0, length(incls))
  Yt[incls == 1] <- Z    
  
  ##############################################
  # Block 1 update
  mui <- (D[id1]*mu[id1]+rho*W[id1,]%*%(phi-mu))/D[id1]  
  s2i <- 1/(lam*D[id1])
  O1 <- incls[id1]
  pm <- as.vector((s2i*Yt[id1]*O1 + mui)/(1 + s2i*O1))
  ps <- as.vector(sqrt((s2i)/(1 + s2i*O1)))
  phi1 <- rnorm(n.id1, pm, ps)
  phi[id1]<-phi1
  
  
  
  ##############################################
  # Block 2 update
  mui <- (D[id2]*mu[id2]+rho*W[id2,]%*%(phi-mu))/D[id2]  
  s2i <- 1/(lam*D[id2])
  O2 <- incls[id2]
  pm <- as.vector((s2i*Yt[id2]*O2 + mui)/(1 + s2i*O2))
  ps <- as.vector(sqrt((s2i)/(1 + s2i*O2)))
  phi2 <- rnorm(n.id2, pm, ps)    
  phi[id2] <- phi2
  
  ##############################################
  # Block 3 update
  mui <- (D[id3]*mu[id3]+rho*W[id3,]%*%(phi-mu))/D[id3]  
  s2i <- 1/(lam*D[id3])
  O3 <- incls[id3]
  pm <- as.vector((s2i*Yt[id3]*O3 + mui)/(1 + s2i*O3))
  ps <- as.vector(sqrt((s2i)/(1 + s2i*O3)))
  phi3 <- rnorm(n.id3, pm, ps)
  phi[id3] <- phi3
  
  ##############################################
  # Block 4 update
  
  mui <- (D[id4]*mu[id4]+rho*W[id4,]%*%(phi-mu))/D[id4]  
  s2i <- 1/(lam*D[id4])
  O4 <- incls[id4]
  pm <- as.vector((s2i*Yt[id4]*O4 + mui)/(1 + s2i*O4))
  ps <- as.vector(sqrt((s2i)/(1 + s2i*O4)))
  phi4 <- rnorm(n.id4, pm, ps)
  phi[id4] <- phi4
  
  
  return(phi)
  
}  # End fn



sampleEta <- function(lamj.SeR, lamj.SpR, mu.SeR, mu.SpR, Zj.SeR, Zj.SpR, Dj, W, 
                      Ts, R, rho, eta.SeR, eta.SpR, id1, id2, id3, id4, n.id1,
                      n.id2, n.id3, n.id4){
  ######################################################
  # This functions samples both the sens. spatial field (phi) and the spec.
  # spatial field (eta) in the model, using the chromatic sampling trick. The
  # chromatic update is in FS.GMRF.
  # 
  # lamg.SeR = precisions associated with the phi_r CAR models
  # lamj.SpR = precisions associated with the eta_r CAR models
  # mu.SeR = mean functions associated with the phi_r CAR models
  # mu.SpR = mean functions associated with the eta_r CAR models
  # Zj.SeR = latent vars associated with the sens. processes
  # Zj.SpR = latent vars associated with the spec. processes
  # Dj = diagonal weight matrix associated with each CAR model (the same for phi
  #   and eta)
  # W = indcidence matrix associated with the CAR models (the same for each)
  # Ts = vector of 'true' inclusion indicators
  # R = number of atlas segmentations
  # rho = propriety parameter for the CAR models
  # eta.SeR = current state of the sens. fields (= phi)
  # eta.SpR = current state of the spec. fields (= eta)
  # id1, ..., id4 = vectors of voxel labels identifying 'colors' of the Markov 
  #   graph
  # n.id1, ..., n.id4 = lengths of the id1, ..., id4 vectors (number of voxels
  #   assigned to each color.)
  
  
  eta.SeR <- lapply(1:R, function(x) FS.GMRF(Zj.SeR[[x]], rho, Dj, W, 
                                             eta.SeR[[x]],
                                             mu.SeR[[x]], lamj.SeR[[x]], id1, 
                                             id2,
                                             id3, id4, n.id1, n.id2, n.id3, 
                                             n.id4,
                                             (Ts == 1)))
  
  eta.SpR <- lapply(1:R, function(x) FS.GMRF(Zj.SpR[[x]], rho, Dj, W, 
                                             eta.SpR[[x]],
                                             mu.SpR[[x]], lamj.SpR[[x]], id1, 
                                             id2,
                                             id3, id4, n.id1, n.id2, n.id3, 
                                             n.id4,
                                             (Ts == 0)))
  
  return(list('eta.SeR'=eta.SeR, 'eta.SpR'=eta.SpR))
  
}  # End fn



sampleLamj<-function(alam.SeR, alam.SpR, blam.SeR, blam.SpR, eta.SeR, eta.SpR, 
                     mu.SeR, mu.SpR, DjWj, R, V){
  ######################################################
  # This function updates the precision parameters (lam_j's) for the spatial
  # processes. 
  #
  # alam.SeR = prior shape parameter for the sens. precisions (these can be
  #   atlas-specific)
  # alam.SpR = prior shape parameter for the spec. precisions
  # blam.SeR = prior rate parameter for the sens. precisions
  # blam.SpR = prior rate parameter for the spec. precisions
  # eta.SeR = spatial fields associated with the sens. processes
  # eta.SpR = spatial fields associated with the spec. processes
  # mu.SeR = mean functions for the sens. spatial processes
  # mu.SpR = mean functions of the specs. spatial processes
  # DjWj = Dj-rho*Wj (= precision matrix for the CAR models)
  # R = number of atlas segmentations
  # V = number of voxels
  
  # Format the shape / rate parameters into lists for updating. These hyper-
  # parameters are specified to (possibly) be atlas-specific
  alam.u.SeR<-lapply(1:R, function(x) alam.SeR[[x]]+V/2)
  alam.u.SpR<-lapply(1:R, function(x) alam.SpR[[x]]+V/2)
  
  
  blam.u.SeR<-lapply(1:R, function(x) as.vector(blam.SeR[[x]] + 
                            t(eta.SeR[[x]]-mu.SeR[[x]])%*%(DjWj)%*%(eta.SeR[[x]]
                                                              -mu.SeR[[x]])/2))
  blam.u.SpR<-lapply(1:R, function(x) as.vector(blam.SpR[[x]] + 
                            t(eta.SpR[[x]]-mu.SpR[[x]])%*%(DjWj)%*%(eta.SpR[[x]]
                                                              -mu.SpR[[x]])/2))
  
  lamj.SeR<-lapply(1:R, function(x) rgamma(1, shape=alam.u.SeR[[x]], 
                                           rate=blam.u.SeR[[x]]))
  lamj.SpR<-lapply(1:R, function(x) rgamma(1, shape=alam.u.SpR[[x]], 
                                           rate=blam.u.SpR[[x]]))
  
  return(list('lamj.SeR'=lamj.SeR, 'lamj.SpR'=lamj.SpR))
  
}  # End fn


sampleTs <- function(Xb, eta.SeR, eta.SpR, Y, V, R){  
  ######################################################
  # Sampling latent T's using the conditional Bernoulli distributions. See the
  # manuscript supplementary material for the parameter specifications, which
  # involve the probit-transformed sens. and spec. processses.
  #
  # Xb = systematic mean function
  # eta.SeR = sens. spatial process realizations
  # eta.SpR = spec. spatial process realizations
  # Y = vector of observed (binary) segmentations
  # V = number of voxels
  # R = number of atlas segmentations
  
  
  p <- exp(Xb)/(1+exp(Xb))  
  
  SeR <- lapply(1:R, function(x) pnorm(eta.SeR[[x]]))  
  Se <- do.call("cbind", SeR)  # This evaluates a function given by the string
  # using the list of arguments provided as the second input. 
  
  SpR <- lapply(1:R, function(x) pnorm(eta.SpR[[x]]))  
  Sp <- do.call("cbind", SpR)
  
  n <- apply(cbind(Se^Y, (1-Se)^(1-Y)), 1, prod)
  d <- apply(cbind(Sp^(1-Y), (1-Sp)^Y), 1, prod)
  q <- p*n/(p*n+(1-p)*d)
  Ts <- rbinom(V, 1, q)
  
  return(list(Ts= Ts, prob= q))  # Returning both T and the probability allows
  # us to use the RB estimator of inclusion
  
}  # End fn


# This function can be helpful if one decides to model the mean functions of the
# sens./spec. processes. By default, though, they modeled with pure spatial
# processes and so have mean zero. In that case there's nothing to update
# and this function is not necessary.
sampleMuSPE<-function(eta.SeR, eta.SpR, lamj.SeR, lamj.SpR, DjWj, Mi, MR, R){
  ######################################################
  # Sampling alpha coefficients for linear predictor for the mean of the spatial
  # random effects phi
  #
  # eta.SeR = sens. spatial field
  # eta.SpR = spec. spatial field
  # lamj.SeR = sens. precision
  # lamj.SpR = spec. precision
  # DjWj = Dj - \rho*W matrix
  # Mi = ??
  # MR = ??
  # R = number of atlases
  
  
  alpha.SeR.mean<-list()
  alpha.SeR.Prec<-list()
  alpha.SeR.Sigma<-list()
  alpha.SeR<-list()
  
  alpha.SpR.mean<-list()
  alpha.SpR.Prec<-list()
  alpha.SpR.Sigma<-list()
  alpha.SpR<-list()
  
  for(r in 1:R){
    
    alpha.SeR.mean[[r]] <- as.vector(as.matrix(solve(t(MR[[r]])%*%DjWj%*%MR[[r]] 
                                                     +
                    (Mi/lamj.SeR[[r]])))%*%t(MR[[r]])%*%t(DjWj)%*%eta.SeR[[r]])
    
    alpha.SeR.Prec[[r]] <- t(MR[[r]])%*%DjWj%*%MR[[r]] + Mi/lamj.SeR[[r]]
    
    alpha.SeR.Sigma[[r]] <- as.matrix(solve(alpha.SeR.Prec[[r]]))
    
    alpha.SeR[[r]] <- rmvnorm(1, alpha.SeR.mean[[r]], alpha.SeR.Sigma[[r]])
    
    
    
    alpha.SpR.mean[[r]] <- as.vector(as.matrix(solve(t(MR[[r]])%*%DjWj%*%MR[[r]] 
                                                     +
                    (Mi/lamj.SpR[[r]])))%*%t(MR[[r]])%*%t(DjWj)%*%eta.SpR[[r]])
    
    alpha.SpR.Prec[[r]] <- t(MR[[r]])%*%DjWj%*%MR[[r]] + Mi/lamj.SpR[[r]]
    
    alpha.SpR.Sigma[[r]]<- as.matrix(solve(alpha.SpR.Prec[[r]]))
    
    alpha.SpR[[r]] <- rmvnorm(1, alpha.SpR.mean[[r]], alpha.SpR.Sigma[[r]])
    
  }
  
  # Means for spatial random effects
  mu.SeR <- lapply(1:R, function(x) MR[[x]]%*%as.vector(alpha.SeR[[x]])) 
  mu.SpR<-lapply(1:R, function(x) MR[[x]]%*%as.vector(alpha.SpR[[x]]))
  
  return(list('mu.SeR'= mu.SeR, 'mu.SpR'= mu.SpR, 'alpha.SeR'= alpha.SeR, 
              'alpha.SpR'= alpha.SpR))
}
