#
#
# SingleBrainImages.R
#
# This reproduces the figures from the single brain ADNI analysis in Section 4
# of the manuscript. 
#
################################################################################

rm(list= ls())

require(neurobase)
require(coda)
require(tidyverse)

# Data directory
brain <- "0592"  # Indicates the brain label of interest
tgt.dir <- paste0("ADNI_data_3D/", brain, "_3D/")  # Where the target structure
# is located

# File with the MCMC results that include gray matter
output.dir <- paste0("Results/", brain, "MCMC_results.Rdata")

# File with MCMC results that did NOT include gray matter
output.dir.nogm <- paste0("Results/", brain, "_NoGM_MCMC_results.Rdata")


img.dir <- "Images/ADNI_Study/"  # Address for the images

# Load the MCMC results
load(output.dir)

# Load the target image
tru.seg <- readNIfTI(paste0(tgt.dir, brain, "_true_segmentation.nii.gz"))

################################################################################

# Print manual segmentations
# This produces Figure 7 in the manuscript
par(bg= "white")
ar <- dim(tru.seg[, , 11])[2]/dim(tru.seg[, , 11])[1]
#x11()
pdf(paste0(img.dir, "3dManualSegementations.pdf"), width= 21, height= 21)
split.screen(c(3,1))  # 3 rows
split.screen(c(1, 3), screen= 1)  # split each row into further screens
split.screen(c(1, 3), screen= 2)
split.screen(c(1, 3), screen= 3)

screen(5)
image(tgt.std[, , 14], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n', 
      bty= 'n', useRaster= TRUE, asp= ar)
image(tru.seg[, , 14], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)  # overlay the segmentation

scrns <- 7:12
for (atlas in 1:6){
  
  temp <- array(Y[, atlas], dim= dim(tgt.std))
  
  screen(scrns[atlas])
  image(tgt.std[ , , 14], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
        bty= 'n', useRaster= TRUE, asp= ar)
  image(temp[, , 14], 
        col=heat.colors(100,alpha=0.5)[100:1], add=TRUE, useRaster=TRUE)  
  
}
dev.off()


################################################################################

# Print covariates

# This produces the left, middle, and right panels of Supplementary Figure 4
# in the manuscript. 

# Manual segmentation
png(paste0(img.dir, "ManSegOrtho.png"))
ortho2(tgt.slc, y= tru.seg, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101))
dev.off()

# SDL map
png(paste0(img.dir, "SDLOrtho.png"))
ortho2(tgt.slc, y= tot.scd, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= TRUE)
dev.off()

# GM Indicator
png(paste0(img.dir, "GMIndOrtho.png"))
ortho2(tgt.slc, y= gm.ind, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= FALSE)
dev.off()


################################################################################

# Print diagnostics
keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])


# This produces Supplementary Figure 5 in the manuscript.
pdf(paste0(img.dir, "3d_Diagnostics.pdf"), width= 28, height= 14)
par(mfrow= c(2,4))
geweke.diag(as.mcmc(beta.MCMC[keep, 1]))
autocorr(as.mcmc(beta.MCMC[keep, 1]))
plot(beta.MCMC[keep, 1], type= "l", xlab= "Iteration", ylab= 
       expression(delta[0]))
mtext("Z = 0.725, Lag 1 Autocorr = 0.328", 3)


geweke.diag(as.mcmc(beta.MCMC[keep, 2]))
autocorr(as.mcmc(beta.MCMC[keep, 2]))
plot(beta.MCMC[keep, 2], type= "l", xlab= "Iteration", ylab= 
       expression(delta[1]))
mtext("Z = 0.062, Lag 1 Autocorr = -0.072", 3)


geweke.diag(as.mcmc(beta.MCMC[keep, 3]))
autocorr(as.mcmc(beta.MCMC[keep, 3]))
plot(beta.MCMC[keep, 3], type= "l", xlab= "Iteration", ylab= 
       expression(delta[2]))
mtext("Z = -0.686, Lag 1 Autocorr = 0.299", 3)

geweke.diag(as.mcmc(beta.MCMC[keep, 4]))
autocorr(as.mcmc(beta.MCMC[keep, 4]))
plot(beta.MCMC[keep, 4], type= "l", xlab= "Iteration", ylab= 
       expression(delta[3]))
mtext("Z = -0.205, Lag 1 Autocorr = 0.062", 3)


beta.intAvg <- cumsum(beta.MCMC[keep, 1])/1:length(beta.MCMC[keep, 1])
beta.itstyAvg <- cumsum(beta.MCMC[keep, 2])/1:length(beta.MCMC[keep, 2])
beta.distAvg <- cumsum(beta.MCMC[keep, 3])/1:length(beta.MCMC[keep, 3])
beta.XAvg <- cumsum(beta.MCMC[keep, 4])/1:length(beta.MCMC[keep, 4])


plot(beta.intAvg, type= "l", xlab= "Iteration", 
     ylab= bquote(delta[1] ~ "Cumulative Average"), main= "")
plot(beta.itstyAvg, type= "l", xlab= "Iteration",
     ylab= bquote(delta[2] ~ "Cumulative Average"), main= "")
plot(beta.distAvg, type= "l", xlab= "Iteration",
     ylab= bquote(delta[3] ~ "Cumulative Average"), main= "")
plot(beta.XAvg, type= "l", xlab= "Iteration", 
     ylab= bquote(delta[4] ~ "Cumulative Average"), main= "")
dev.off()


################################################################################

# Posterior prob. map and uncertainty
prb.mean <- apply(probs.MCMC[keep, ], 2, mean)
prb.se <- apply(probs.MCMC[keep, ], 2, sd)

# Run the commented code to view only one slice
# x11(height= 7, width= 14)
# par(mfrow= c(1,2))
# ar <- dim(tru.seg[, , 11])[2]/dim(tru.seg[, , 11])[1]
# #ar <- dim(tgt.std.0016)[2]/dim(tgt.std.0016)[1]
# image(tgt.std[ , ,11], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n', bty= 'n',
#       useRaster= TRUE, asp= ar, main= "Inclusion Probabilities")
# image(array(prb.mean, c(P1, P2, P3))[, , 11], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
#       useRaster=TRUE) 
# 
# 
# image(tgt.std[ , , 11], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n', bty= 'n',
#       useRaster= TRUE, asp= ar, main= "Standard Deviations")
# # smallplot = (left side point, right side point, bottom side point, top side point)
# image.plot(array(prb.se, c(P1, P2, P3))[, , 11], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
#            useRaster=TRUE, zlim= c(0,1), smallplot= c(0.005, 0.020, 0.2, 0.85))


# 3d depiction
# The produces the left and right panels of Figure XX in the manuscript
prb.mn.nifti <- as.nifti(array(prb.mean, c(P1, P2, P3)))
prb.se.nifti <- as.nifti(array(prb.se, c(P1, P2, P3)))
png(paste0(img.dir, "PostMeanOrtho.png"))
ortho2(tgt.slc, y= prb.mn.nifti, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= TRUE)
dev.off()


png(paste0(img.dir, "PostSDOrtho.png"))
ortho2(tgt.slc, y= prb.se.nifti, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= TRUE)
dev.off()

################################################################################


# Below we can possibly compare with competing LF procedures, and also show
# GM vs. No GM results. Think about this more. See AdditionalBLFFigs_R3.R
# to yank the code for that stuff.

# Get additional segmentations to compare

# Majority voting
maj.vot <- apply(segmentations, c(1,2,3), sum)
maj.vot <- (maj.vot > dim(segmentations)[4]/2)


# Globally-weighted MV
gw.maj.vot1 <- array(0, c(P1, P2, P3))
gw.maj.vot0 <- array(0, c(P1, P2, P3))
for (i in 1:dim(segmentations)[4]){
  
  gw.maj.vot1 <- gw.maj.vot1 + segmentations[, , , i]/mean(sims[, , , i]^2)
  gw.maj.vot0 <- gw.maj.vot0 + (1-segmentations[, , , i])/mean(sims[, , , i]^2)
  
}
gw.maj.vot <- array(0, c(P1, P2, P3))
for (i in 1:P1){
  for (j in 1:P2){
    for(k in 1:P3){
      
      gw.maj.vot[i,j,k] <- ifelse(gw.maj.vot1[i,j,k] > gw.maj.vot0[i,j,k], 1, 0)  
      
    }
  }
}
gw.maj.vot[gw.maj.vot1 == gw.maj.vot0] <- rbinom(1, 1, 0.5)


# Locally-weighted MV
lw.maj.vot1 <- apply(segmentations/(sims^2), c(1,2,3), sum)
lw.maj.vot0 <- apply((1-segmentations)/(sims^2), c(1,2,3), sum)
lw.maj.vot <- array(0, c(P1, P2, P3))
for (i in 1:P1){
  for (j in 1:P2){
    for (k in 1:P3){
      
      lw.maj.vot[i,j,k] <- ifelse(lw.maj.vot1[i,j,k] > lw.maj.vot0[i,j,k], 1, 0)   
      
    }
  }
}
lw.maj.vot[lw.maj.vot1 == lw.maj.vot0] <- rbinom(1, 1, 0.5)


# Load joint label fusion result
jlf.seg <- readNIfTI(paste0(tgt.dir, brain, 
                            "_jlf_segmentation_cn_to_ad.nii.gz"))

# Threshold Bayesian LF to get a segmentation
blf.est <- (array(prb.mean, c(P1, P2, P3)) > 0.5)*1

################################################################################

# Tabulate and compute volume similarities
tab.blf <- table(c(tru.seg), blf.est)
tab.jlf <- table(c(tru.seg), jlf.seg)
tab.mv <- table(c(tru.seg), maj.vot)
tab.gwv <- table(c(tru.seg), gw.maj.vot)
tab.lwv <- table(c(tru.seg), lw.maj.vot)

vs_blf <- 1 - abs(tab.blf[2,1] - tab.blf[1,2])/(2*tab.blf[2,2] + tab.blf[1,2] +
                                                  tab.blf[2,1])


vs_jlf <- 1 - abs(tab.jlf[2,1] - tab.jlf[1,2])/(2*tab.jlf[2,2] + tab.jlf[1,2] +
                                                  tab.jlf[2,1]) # 0.997


vs_mv <- 1 - abs(tab.mv[2,1] - tab.mv[1,2])/(2*tab.mv[2,2] + tab.mv[1,2] +
                                               tab.mv[2,1]) # 0.825


vs_gwv <- 1 - abs(tab.gwv[2,1] - tab.gwv[1,2])/(2*tab.gwv[2,2] + tab.gwv[1,2] +
                                                  tab.gwv[2,1]) # 0.764


vs_lwv <- 1 - abs(tab.lwv[2,1] - tab.lwv[1,2])/(2*tab.lwv[2,2] + tab.lwv[1,2] +
                                                  tab.lwv[2,1])  # 0.801


# Plot one 2d slice of the segmentations with volume similarities

# This creates Figure 9 in the manuscript
pdf(paste0(img.dir, "SingleSliceSegResults.pdf"), width= 21, height= 14)
par(mfrow= c(2,3))
ar <- dim(tru.seg[, , 12])[2]/dim(tru.seg[, , 12])[1]

image(tgt.std[ , , 12], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
      useRaster= TRUE, asp= ar, main= "Manual", cex.main= 3.5, bty= "n")
image(tru.seg[ , , 12], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)  # overlay the segmentation

image(tgt.std[, , 12], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
      useRaster= TRUE, asp= ar, 
      main= paste0("BLF (VS =", round(vs_blf, 3), ")"), cex.main= 3.5,
      bty= "n")
image(blf.est[ , , 12], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)  

image(tgt.std[ , , 12], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
      useRaster= TRUE, asp= ar, 
      main= paste0("JLF (VS =", round(vs_jlf, 3), ")"), cex.main= 3.5,
      bty = "n")
image(jlf.seg[ , , 12], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)  

image(tgt.std[ , , 12], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
      useRaster= TRUE, asp= ar, 
      main= paste0("LW Maj. Vote (VS =", round(vs_lwv, 3), ")"), cex.main= 3.5,
      bty= "n")
image(lw.maj.vot[ , , 12], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)

image(tgt.std[ , , 12], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
      useRaster= TRUE, asp= ar, 
      main= paste0("GW Maj. Vote (VS =", round(vs_gwv, 3), ")"), cex.main= 3.5,
      bty= "n")
image(gw.maj.vot[ , , 12], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)

image(tgt.std[ , , 14], col=grey(seq(0, 1, length.out=100)), xaxt='n', yaxt='n',
      useRaster= TRUE, asp= ar, 
      main= paste0("Maj. Vote (VS =", round(vs_mv, 3), ")"), cex.main= 3.5,
      bty= "n")
image(maj.vot[ , , 12], col=heat.colors(100,alpha=0.5)[100:1], add=TRUE,
      useRaster=TRUE)
dev.off()

################################################################################

# Illustrate sensitivity and specificity on one Atlas
# This produces the four individual panels of Supplementary Figure 6 
# in the manuscript
SeR.mn.array <- array(unlist(eta.SeR.mean), dim= c(P1, P2, P3, R))
SpR.mn.array <- array(unlist(eta.SpR.mean), dim= c(P1, P2, P3, R))

png(paste0(img.dir, "ManSegOrthoWithLabel.png"))
ortho2(tgt.slc, y= tru.seg, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101),
       text= "Manual Segmentation")
dev.off()

png(paste0(img.dir, "Atlas6Segmentation.png"))
ortho2(tgt.slc, y=Yv[ , , , 6], col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101),
       text= "Atlas 6 Segmentation")
dev.off()

png(paste0(img.dir, "Atlas6Sens.png"))
ortho2(tgt.slc, y=SeR.mn.array[, , , 6], col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= TRUE,
       text= "Sensitivity Field")
dev.off()

png(paste0(img.dir, "Atlas6Spec.png"))
ortho2(tgt.slc, y=SpR.mn.array[, , , 6], col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= TRUE,
       text= "Specificity Field")
dev.off()

################################################################################

# Compare the BLF with and without gray matter information
rm(list= ls()[!(ls() %in% c('tgt.dir', 'output.dir', 'output.dir.nogm', 
                            'img.dir'))])
load(output.dir.nogm)

keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])
prb.mean <- apply(probs.MCMC[keep, ], 2, mean)
prb.mn.nifti <- as.nifti(array(prb.mean, c(P1, P2, P3)))

# This plot, combined with the manual segmentation orthographic and the
# previous posterior probability map orthographic make the three panels in 
# Figure 10 in the manuscript.
png(paste0(img.dir, "PostMean_NoGM_Ortho.png"))
ortho2(tgt.slc, y= prb.mn.nifti, col.y= heat.colors(100,alpha=0.5)[100:1],
       ybreaks= seq(0, 1, length= 101), ycolorbar= TRUE)
dev.off()

################################################################################

# Plot the volume estimates
rm(list= ls()[!(ls() %in% c('tgt.dir', 'output.dir', 'output.dir.nogm', 
                            'img.dir'))])
load(output.dir)
tru.seg <- readNIfTI(paste0(tgt.dir, brain, "_true_segmentation.nii.gz"))

####


# Get the original volume estimates and other segmentations (again)
keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])
vol.ests.rb <- apply(probs.MCMC[keep, ], 1, sum)


# Majority voting
maj.vot <- apply(segmentations, c(1,2,3), sum)
maj.vot <- (maj.vot > dim(segmentations)[4]/2)


# Globally-weighted MV
gw.maj.vot1 <- array(0, c(P1, P2, P3))
gw.maj.vot0 <- array(0, c(P1, P2, P3))
for (i in 1:dim(segmentations)[4]){
  
  gw.maj.vot1 <- gw.maj.vot1 + segmentations[, , , i]/mean(sims[, , , i]^2)
  gw.maj.vot0 <- gw.maj.vot0 + (1-segmentations[, , , i])/mean(sims[, , , i]^2)
  
}
gw.maj.vot <- array(0, c(P1, P2, P3))
for (i in 1:P1){
  for (j in 1:P2){
    for(k in 1:P3){
      
      gw.maj.vot[i,j,k] <- ifelse(gw.maj.vot1[i,j,k] > gw.maj.vot0[i,j,k], 1, 0)  
      
    }
  }
}
gw.maj.vot[gw.maj.vot1 == gw.maj.vot0] <- rbinom(1, 1, 0.5)


# Locally-weighted MV
lw.maj.vot1 <- apply(segmentations/(sims^2), c(1,2,3), sum)
lw.maj.vot0 <- apply((1-segmentations)/(sims^2), c(1,2,3), sum)
lw.maj.vot <- array(0, c(P1, P2, P3))
for (i in 1:P1){
  for (j in 1:P2){
    for (k in 1:P3){
      
      lw.maj.vot[i,j,k] <- ifelse(lw.maj.vot1[i,j,k] > lw.maj.vot0[i,j,k], 1, 0)   
      
    }
  }
}
lw.maj.vot[lw.maj.vot1 == lw.maj.vot0] <- rbinom(1, 1, 0.5)


# Load joint label fusion result
jlf.seg <- readNIfTI(paste0(tgt.dir, brain, 
                            "_jlf_segmentation_cn_to_ad.nii.gz"))



# # This reproduces Figure 11 in the manuscript

## The commented out code will produce a black and white version of the figure.
## The uncommented code produces a color figure via ggplot

# pdf(paste0(img.dir, "VolumeDensities.pdf"))
# plot(density(vol.ests.rb), lwd= 2, xlim= c(2500, 4500),
#      xlab= bquote("Volume"~"(mm"^3~")"),
#      ylim= c(0, 0.0125),
#      main= "")
# abline(v= sum(tru.seg), lwd= 2)
# abline(v= sum(jlf.seg), lwd= 2, lty= 2)
# abline(v= sum(lw.maj.vot), lwd= 2, lty= 3)
# abline(v= sum(gw.maj.vot), lwd= 2, lty= 4)
# abline(v= sum(maj.vot), lwd= 2, lty= 5)
# legend("topright", legend= c("Manual", "JLF", "LW Maj. Vote", "GW Maj. Vote",
#                                   "Maj. Vote"), lty= 1:5, bg= "white")
# 
# 
# 
# 
# 
# # Get the volume estimates without GM segmentation and add to plot
# rm(list= ls()[!(ls() %in% c('tgt.dir', 'output.dir', 'output.dir.nogm', 
#                             'img.dir'))])
# load(output.dir.nogm)
# 
# keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])
# vol.ests.rb <- apply(probs.MCMC[keep, ], 1, sum)
# 
# lines(density(vol.ests.rb), lty= 6)
# text(x= 3000, y= 0.01, labels= "With Gray Matter", bty= "n", cex= 0.75)
# text(x= 3400, y= 0.007, labels= "Without Gray Matter", bty= "n", cex= 0.75)
# dev.off()

man_vol <- sum(tru.seg)
jlf_vol <- sum(jlf.seg)
lwmv_vol <- sum(lw.maj.vot)
gwmv_vol <- sum(gw.maj.vot)
mv_vol <- sum(maj.vot)

blf_vol_dens <- data.frame(gm_vol= vol.ests.rb)


#################

# Get the volume estimates without gray matter
rm(list= ls()[!(ls() %in% c('tgt.dir', 'output.dir', 'output.dir.nogm', 
                            'img.dir', 'man_vol', 'jlf_vol', 'lwmv_vol',
                            'gwmv_vol', 'mv_vol', 'blf_vol_dens'))])
gc()

load(output.dir.nogm)

keep <- (length(lamj.SeR.MCMC[[1]])/2 + 1):length(lamj.SeR.MCMC[[1]])
vol.ests.rb <- apply(probs.MCMC[keep, ], 1, sum)

blf_vol_dens$nogm_vol <- vol.ests.rb
type <- c(rep("Gray Matter", times= dim(blf_vol_dens)[1]), 
          rep("No Gray Matter", times= dim(blf_vol_dens)[1]))

# Stack the volumes
temp <- c(blf_vol_dens$gm_vol, blf_vol_dens$nogm_vol)
blf_vols <- data.frame(Vol= temp, BLF= type)

# Stack the other segmentations
seg_type <- c("Manual", "JLF", "LW Maj. Vote", "GW Maj. Vote", "Maj. Vote")
seg_vol <- c(man_vol, jlf_vol, lwmv_vol, gwmv_vol, mv_vol)
segs_df <- data.frame(Volume= seg_vol, Segmentation= seg_type)

# This reproduces Figure 11 in the manuscript
pdf(paste0(img.dir, "VolumeDensities_Pretty.pdf"), height= 7, width= 11)
ggplot(blf_vols, aes(x = Vol)) + 
  xlim(2500, 4500) +
  xlab(bquote("Volume"~"(mm"^3~")")) +
  ylab("Density") +
  geom_density(aes(fill = BLF), alpha= 0.3) +
  geom_vline(aes(xintercept = Volume, color= Segmentation, 
                 linetype= Segmentation), size= 0.75,
                 data= segs_df)
dev.off()
