#
# SingleBrain_MCMC.R
#
# This sets up and runs the Bayesian label fusion model for the in-depth
# analysis of a single AD brain (labeled 0592), using gray matter indicators
# as a covariate. The results are saved to be loaded in by a separate script
# for summarizing the results.
#-------------------------------------------------------------------------------

rm(list= ls())

require(Rcpp)
require(RcppTN)  # For truncated normal in CPP
require(RcppArmadillo)  # Numerical linear algebra library
require(parallel)
require(Matrix)
require(oro.nifti)
require(R.matlab)


# Compile the MCMC code
sourceCpp("BLF_MCMCAuxiliary.cpp")

# Open parallel cluster and set up each worker
cl <- makeCluster(detectCores()-1)
clusterEvalQ(cl, library(Matrix))
clusterEvalQ(cl, library(Rcpp))
clusterEvalQ(cl, library(RcppTN))
clusterEvalQ(cl, library(RcppArmadillo))
clusterEvalQ(cl, sourceCpp("BLF_MCMCAuxiliary.cpp"))  # To parLapply Rcpp fns


# Set seeds for reproducibility
set.seed(123)
seeds <- c(234, 345, 456, 567, 678, 789, 890)  # assumes 8-1 = 7 cores
clusterApply(cl, seeds, set.seed)


# Data directory
brain <- "0592"  # Indicates the brain label of interest
tgt.dir <- paste0("ADNI_data_3D\\", brain, "_3D\\")
labs <- c("0016", "0413", "0419", "0525", "0067", "0172")  # Rater image labels


# Results directory
res.dir <- paste0("Results\\", brain, "MCMC_results.Rdata")


# No. iterations and thinning for the MCMC
iter <- 150000
thin <- 50



#-------------------------------------------------------------------

# Read in the data

# White-stripe-normalized T1 target image
tgt.slc <- readNIfTI(paste0(tgt.dir, brain, "_target_slice.nii.gz"))

# Tissue class segmentation
tissue <- readNIfTI(paste0(tgt.dir, brain, "_tissue_segmentation.nii.gz"))
gm.ind <- (tissue == 2)*1  # Gray matter indicator



# Get rater images, registered T1s, and similarity images
Yv <- array(dim= c(dim(tgt.slc), 6))  # Atlas segmentations
t1s <- array(dim= c(dim(tgt.slc), 6))  # registered t1 images
sims <- array(dim= c(dim(tgt.slc), 6))  # similarity images (eucliden norms)


pos <- 1
# Storing the nifti's coerces them to arrays, since Yv etc. are arrays
for(ind in labs[labs != brain]){
  
  Yv[, , , pos] <- readNIfTI(paste0(tgt.dir, ind, "_atlas_segmentation.nii.gz"))
  t1s[, , , pos] <- readNIfTI(paste0(tgt.dir, ind, "_atlas_t1.nii.gz"))
  sims[, , , pos] <- readNIfTI(paste0(tgt.dir, ind, "_similarity.nii.gz"))
  
  pos <- pos + 1
  
}  # End read-in loop




#-----------------------------------------------------------------

# Construct covariates with assistance from MATLAB

# For each atlas segmentation, get the signed distance label transform via
# MATLAB:
Matlab$startServer()  # Using MATLAB R2018a
matlab <- Matlab()  
isOpen <- open(matlab)  

setVariable(matlab, Yv= Yv)
mat.com <- paste0("for i=1:6 Yv_sDist(:,:,:,i)= bwdist(Yv(:,:,:,i)) - ",
                  "bwdist(1-Yv(:,:,:,i)); end")

evaluate(matlab, mat.com)  # The runs the code in MATLAB

# Return the result and close MATLAB
Yv.sDist <- getVariable(matlab, "Yv_sDist")
Yv.sDist <- Yv.sDist[[1]]
close(matlab)



# Create a covariate via weighted sum of the signed distance labels. Weight them
# by the inverse sq. norm of voxel intensity difference.
tot.sDist <- apply(Yv.sDist/(sims^2), c(1,2,3), sum)/apply(1/(sims^2), c(1,2,3),
                                                           sum)

# NOW rescale them to the unit interval
tot.scd <- (tot.sDist - min(tot.sDist))/(max(tot.sDist) - min(tot.sDist))

# Rescale the target intensities to [0,1] as well
tgt.std <- (tgt.slc - min(tgt.slc))/(max(tgt.slc) - min(tgt.slc))




#--------------------------------------------------------------------

# Set up the model 

segmentations <- Yv  # This the array of segmentations

R <- dim(segmentations)[4]  # R = available number of rater images
M <- 1  # M = #coeffs. in the mean functions of the Se/Sp processes, including
# intercept.
P <- 4  # P = # coeffs. in the mean function for the latent inclusion indicators,
# including intercept.

P1 <- dim(segmentations)[1]
P2 <- dim(segmentations)[2]
P3 <- dim(segmentations)[3]

V <- P1*P2*P3  # V = total number of voxels
Vid <- array(1:V, c(P1,P2,P3)) # Identifies voxels  
Y <- matrix(-99, V, R)     # Outcome based on Segmentation R
iSim <- matrix(-99, V, R)  # euclidean distance between atlas R and target image

X <- matrix(nrow=V, ncol=P) # Design matrix for true inclusion model
Dj <- rep(-99, V)  # The voxel-specific "weights" for the CAR model
Nj <- matrix(-99, nrow=V, ncol=26) # This will list the neighbors for each 
# voxel. (26 = 9*3 - 1 for 3D case)


# Need 8 colors for 3D graph
id1 <- rep(-99, V)  # For labeling the "colors" in the Markov graph based on the 
# assumed dependence structure.
id2 <- rep(-99, V)
id3 <- rep(-99, V)
id4 <- rep(-99, V)
id5 <- rep(-99, V)
id6 <- rep(-99, V)
id7 <- rep(-99, V)
id8 <- rep(-99, V)

c1 <- 1  # Counters to keep track of how many we have of each color.
c2 <- 1
c3 <- 1
c4 <- 1
c5 <- 1
c6 <- 1
c7 <- 1
c8 <- 1



# Set up neighborhood structure, colorings for the chromatic sampler, the 
# design matrix, etc.
for(p1 in 1:P1){
  for(p2 in 1:P2){
    for(p3 in 1:P3){
      
      l1 = max(c(1, p1-1))  # Lower bound along dim 1 (rows)
      l2 = max(c(1, p2-1))  # cols
      l3 = max(c(1, p3-1))  # 
      
      u1 = min(c(P1, p1+1))  # Upper bound along dim 1 (rows)
      u2 = min(c(P2, p2+1))  # cols
      u3 = min(c(P3, p3+1))  #
      
      neighbors = as.vector(Vid[l1:u1, l2:u2, l3:u3])  # Vectorize the labels 
      # corresponding to the point of interest.
      
      neighbors = neighbors[neighbors != Vid[p1, p2, p3]]  # Can't be a 
      # neighbor with yourself!
      
      wjv = length(neighbors)  # The w_j weights = number of neighbors for 
      # voxel j
      
      Dj[Vid[p1, p2, p3]] = wjv  # Store it in weight matrix
      Nj[Vid[p1, p2, p3], 1:wjv] = neighbors  # Mostly we have the neighbor 
      # labels
      
      
      # Systematically filling in the colors based on row/col/plane combination.
      if(p1%%2!=0 & p2%%2!=0 & p3%%2!=0){ 
        
        id1[c1]<-Vid[p1,p2,p3]
        c1<-c1+1
        
      }  # Odd rows and odd cols
      if(p1%%2!=0 & p2%%2==0 & p3%%2!=0){ 
        
        id2[c2]<-Vid[p1,p2,p3]
        c2<-c2+1
        
      }  # Odd rows and even cols
      if(p1%%2==0 & p2%%2!=0 & p3%%2!=0){ 
        
        id3[c3]<-Vid[p1,p2,p3]
        c3<-c3+1
        
      }  # Even rows and odd cols
      if(p1%%2==0 & p2%%2==0 & p3%%2!=0){ 
        
        id4[c4]<-Vid[p1,p2,p3]
        c4<-c4+1
        
      }  # Even rows and even cols
      
      
      if(p1%%2!=0 & p2%%2!=0 & p3%%2==0){ 
        
        id5[c5]<-Vid[p1,p2,p3]
        c5<-c5+1
        
      }  # Odd rows and odd cols
      if(p1%%2!=0 & p2%%2==0 & p3%%2==0){ 
        
        id6[c6]<-Vid[p1,p2,p3]
        c6<-c6+1
        
      }  # Odd rows and even cols
      if(p1%%2==0 & p2%%2!=0 & p3%%2==0){ 
        
        id7[c7]<-Vid[p1,p2,p3]
        c7<-c7+1
        
      }  # Even rows and odd cols
      if(p1%%2==0 & p2%%2==0 & p3%%2==0){ 
        
        id8[c8]<-Vid[p1,p2,p3]
        c8<-c8+1
        
      }  # Even rows and even cols
      
      
      # Here is where we are storing the covariate info in the design matrix.
      X[Vid[p1, p2, p3], 1] <- 1  # Intercept
      X[Vid[p1, p2, p3], 2] <- tot.scd[p1, p2, p3]  # Weighted sum of SDLs
      X[Vid[p1, p2, p3], 3] <- gm.ind[p1, p2, p3]  # gm indicator as predictor
      X[Vid[p1, p2, p3], 4] <- X[Vid[p1, p2, p3], 2]*X[Vid[p1, p2, p3], 3]  
      
      
      for(r in 1:R){
        
        Y[Vid[p1,p2,p3], r] = Yv[p1, p2, p3, r]  # Simply creating the 
        # array of observed labels
        
      }  # End R loop
      
    }  # End P3 loop
  }  # End P2 loop
}  # End P1 loop



id1 <- id1[id1 != -99]  # Collapse the color vectors to only contain "real" voxel
# labels
id2 <- id2[id2 != -99]
id3 <- id3[id3 != -99]
id4 <- id4[id4 != -99]
id5 <- id5[id5 != -99]
id6 <- id6[id6 != -99]
id7 <- id7[id7 != -99]
id8 <- id8[id8 != -99]

n.id1 <- length(id1)  # Number of voxels corresponding to each color.  
n.id2 <- length(id2)    
n.id3 <- length(id3)
n.id4 <- length(id4)
n.id5 <- length(id5)
n.id6 <- length(id6)
n.id7 <- length(id7)
n.id8 <- length(id8)



# Creating the sparse matrices to store the Laplacian
i = c(NA)
length(i) <- sum(Dj)
j = c(NA)
length(j) <- length(i)

spot <- 1
for(v in 1:V){
  
  i[spot:(spot + Dj[v]-1)] <- rep(v, Dj[v])
  
  j[spot:(spot + Dj[v]-1)] <- Nj[v, 1:Dj[v]]
  
  spot <- spot + Dj[v]
  
}  # End for


rho <- 0.995
Wj <- sparseMatrix(i, j, x=1)  
Dj <- sparseMatrix(1:V, 1:V, x=Dj)
DjWj <- Dj-rho*Wj   # making the prior proper by fixing the propriety parameter!
I <- sparseMatrix(1:V,1:V,x=1)
Dj <- diag(Dj) 



#----------------------------------------------------------------------------

# Prior specification

alam.SeR <- vector("list", R)
alam.SeR[1:R] <- 1 # hyperparameters for the prior on lambda sensitivity model. a is
# the shape, b is the rate

blam.SeR <- vector("list", R)
blam.SeR[1:R] <- 2



alam.SpR <- vector("list", R)
alam.SpR[1:R] <- 1 # hyperparameters for the prior on lambda specificity model.

blam.SpR <- vector("list", R)
blam.SpR[1:R] <- 2



# Specified mean of spatial process
mu.SeR <- lapply(1:R, function(x) rep(0, V))   
mu.SpR <- lapply(1:R, function(x) rep(0, V))






# BCJ prior specification. This is where we can indirectly induce a prior on
# the regression coefficient(s) via CMP elicitation.  Need four conditions: 
# 1. Small distance, GM
# 2. Large distance, ~GM
# 3. Large distance, GM
# 4. Small distance, ~GM
x.add <- rbind(c(1, 0, 1, 0),
               c(1, 0.5, 0, 0),
               c(1, 0.5, 1, 0.5),
               c(1, 0.1, 0, 0))

p.add <- c(0.9999, 0.001, 0.1, 0.01)
n.add <- c(25000, 25000, 25000, 25000)
y.add <- n.add*p.add



#------------------------------------------------------------------------------

# Set up storage and initialize MCMC

# Initialize the spatial processes representing the linear predictor
# part of the probit link model on sensitivity and specificity
eta.SeR <- lapply(1:R, function(x) rep(1, V))  
eta.SpR <- lapply(1:R, function(x) rep(1, V))


# Initialize precisions for sens/spec processes
lamj.SeR <- vector("list", R)
lamj.SeR[1:R] <- 1/2
lamj.SpR <- vector("list", R)
lamj.SpR[1:R] <- 1/2


Ts <- rbinom(V, 1, apply(Y, 1, mean))   #  Use these beta values as the initial
#  state for the Gamerman MH proposal

beta.init <- coef(glm(c(Ts, y.add/n.add) ~ rbind(X, x.add)-1, family= "binomial",
                      weights= c(rep(1, times= length(Ts)), n.add) ))  # Ignore 
# the warning since the data are "fake"


# Storage devices
Ts.MCMC <- matrix(-99, nrow=iter/thin, ncol=V) 
probs.MCMC <- matrix(-99, nrow= iter/thin, ncol= v)

# Instead of storing each iteration of the spatial fields, just store the means
# to preserve memory
# eta.SeR.MCMC <- lapply(1:R, function(x) return(matrix(-99, nrow=iter/thin, 
# ncol=V)))
# eta.SpR.MCMC <- lapply(1:R, function(x) return(matrix(-99, nrow=iter/thin, 
# ncol=V)))
eta.SeR.mean <- lapply(1:R, function(x) return(matrix(-99, nrow=1, ncol=V)))
eta.SpR.mean <- lapply(1:R, function(x) return(matrix(-99, nrow=1, ncol=V)))


lamj.SeR.MCMC <- lapply(1:R, function(x) return(matrix(-99, nrow=iter/thin, 
                                                       ncol=1)))
lamj.SpR.MCMC <- lapply(1:R, function(x) return(matrix(-99, nrow=iter/thin, 
                                                       ncol=1)))

beta.MCMC <- matrix(-99, nrow=iter/thin, ncol=(P))



#------------------------------------------------------------------------------

# Run the Markov chain
beta <- beta.init

clusterExport(cl, varlist= c("FS_GMRF", "rho", "Dj", "Wj", "Nj", "mu.SeR", 
                             "mu.SpR", "id1", "id2", "id3", "id4", "id5",
                             "id6", "id7", "id8"), envir= environment())  
clusterEvalQ(cl, sourceCpp("BLF_MCMCAuxiliary.cpp"))  # I have no idea why, but
# this needs to be run twice

system.time(
  for(i in 1:iter){
    
    ######################################################
    # Sampling beta with a standard N(0,t2*I)
    beta <- c(MH_logistic_reg(b0=beta, X=X, y=Ts, y_add=y.add, n_add=n.add,
                              X_add=x.add))
    
    
    Xb <- X%*%beta
    
    #####################################################
    # Sampling latent Zj
    getZj <- sampleZj_C(eta.SeR, eta.SpR, Ts, Y, R, V)
    Zj.SeR <- getZj$Zj_SeR 
    Zj.SpR <- getZj$Zj_SpR 
    
    
    ######################################################
    # Sampling the spatial random effects phi
    clusterExport(cl, varlist= c("Zj.SeR", "Zj.SpR", "eta.SeR", "eta.SpR",
                                 "lamj.SeR", "lamj.SpR", "Ts"),
                  envir= environment())
    
    
    eta.SeR <- parLapply(cl, 1:R, function(x) FS_GMRF(Zj.SeR[[x]], rho, Dj,
                                                      Nj, mu.SeR[[x]], 
                                                      eta.SeR[[x]],
                                                      lamj.SeR[[x]],
                                                      id1, id2, id3, id4, id5,
                                                      id6, id7, id8,
                                                      (Ts == 1)) )
    
    
    eta.SpR <- parLapply(cl, 1:R, function(x) FS_GMRF(Zj.SpR[[x]], rho, Dj,
                                                      Nj, mu.SpR[[x]], 
                                                      eta.SpR[[x]],
                                                      lamj.SpR[[x]],
                                                      id1, id2, id3, id4, id5,
                                                      id6, id7, id8,
                                                      (Ts == 0)) )
    
    ######################################################
    # Sampling lamj
    getLamj <- Sample_Lamj(alam.SeR, alam.SpR, blam.SeR, blam.SpR, eta.SeR, eta.SpR,
                           mu.SeR, mu.SpR, DjWj, R, V)
    
    lamj.SeR <- getLamj$lamj_SeR
    lamj.SpR <- getLamj$lamj_SpR
    
    ######################################################
    # Sampling latent T's  
    bin.out <- Sample_Ts(Xb, eta.SeR, eta.SpR, Y, V, R)
    
    Ts <- c(bin.out[[1]])
    prb <- bin.out[[2]]
    
    
    ######################################################
    # Store the sample   
    if( i %% thin == 0 ){
      
      j <- i/thin
      Ts.MCMC[j, ] <- Ts
      probs.MCMC[j, ] <- prb
      beta.MCMC[j, ] <- beta
      
      
      
      for(r in 1:R){
        
        #eta.SeR.MCMC[[r]][j,] <-eta.SeR[[r]]
        
        # Rather than storing the mean of the raw (unbounded) GMRFs, update the 
        # mean of the probit-transformed fields for better illustration
        #eta.SeR.mean[[r]][1, ] <- ((j-1)*eta.SeR.mean[[r]][1, ] + eta.SeR[[r]])/j
        eta.SeR.mean[[r]][1, ] <- ((j-1)*eta.SeR.mean[[r]][1, ] + 
                                     pnorm(eta.SeR[[r]]))/j
        lamj.SeR.MCMC[[r]][j,]<-lamj.SeR[[r]]
        
        
        #eta.SpR.MCMC[[r]][j,] <-eta.SpR[[r]]
        #eta.SpR.mean[[r]][1, ] <- ((j-1)*eta.SpR.mean[[r]][1, ] + eta.SpR[[r]])/j
        eta.SpR.mean[[r]][1, ] <- ((j-1)*eta.SpR.mean[[r]][1, ] + 
                                     pnorm(eta.SpR[[r]]))/j
        lamj.SpR.MCMC[[r]][j,]<-lamj.SpR[[r]]
        
      }  # End for
      
    }  # End if
    
    if (i %% 100 == 0){
      print(i)  # Track progress
    }
    
  }  # End MCMC loop
  
)  # End system time -     user      system   elapsed 
#                       33759.63  7460.54  60621.61 

stopCluster(cl)

#-------------------------------------------------------------------------------

# Save the results 
save.image(res.dir)
